; *************************
; * Irrigation controller *
; *************************
;
; What it does:
; System is an irrigation control system for garden-usage
;
; Controls up to 4 zones / solonoid controlled valves
; -depending on signal from a humidity sensor (optional)
; -time controlled (starttime & duration)
; -possibility to switch on each zone manually
;
; Additional zone for a refill control system (refill of a reservoir etc...)
; -including level control
;
; optional: pump relais to switch a pump or a mains valve
; real time clock, battery-buffered
; 4 row display, 6 push buttons
;
; Stephan Asmus, Germany
; info@sasmus.de
; 
; This software is free for personal, non commercial use. 
; You are allowed to distribute this software or reuse parts of it under
; the same conditions. You have to name the original author, however.
; You are not allowed to make profit in reusing this software or parts
; of it.
; Feedback is welcome - however don't assume that individual changes 
; or additions are provided. This is a hobby project which fits _my_ needs
; and time is limited.
;
;----------------------------------------
; Log of development:
; 2011
; Start: 01.03.2011 :Base outlined
; 02.03.: math & converter - routines outlined
; 23.03.: i2C Clock & Display working 
; 24.03.: Mainmenu
; 26.03.: Submenu Program working
; 26.03.: Submenu Refill System working
; 26.03.: Submenu Humidity working
; 26.03.: Submenu Timeset working
; running out of memory in pgm Page 0... need to optimize or transfer to P.1
; 28.03.: Transferred many routines to page1 &implement long call routines
; 29.03.: Preferences load/save done
; 30.03.: Valve & Pump manual operation
; 31.03.: Changed usage: now you have to press >3sec for status manual off
; 06.04.: Problem of using I2C and Display Routines from Interrupt solved (1)
; 06.04.: Valve manual off saved to eeprom to ensure no hassles after power down
; 06.04.: TMR1 init for humidity sensor check
; 09.04.: start pgm run implemented (time, flags, sensor)
; 12.04.: start implementing program logic
; 13.04.: program logic working - moved to external procedure
; 28.04.: Refill system; isr for humidity check
; 02.05.: Main circuit completed and tested -working
; 03.05.: Preparations for Humidity measurement, tmr1 working
; 04.05.: Calibration hum sensor ready
; 05.05.: humidity measurement working
;		: Backlight auto off
;		: Errorflags and display of them in status-line
; 14.05.: Some minor error corrections
; 		: ready to go
; 18.05.: hard coded goto/return of tmr0isr to save one stack level
;		: corrected manual stop condition to execute immediately
; 23.05.: Some error corrections
; 15.06.: experimental display_on bevor updating display in main routine
;		  Display/pump status corrected when refill =on
; 16.06.: extended experimental display_on to correct display init
;		  prolonged time after RS set in display write routine
; 2014
; 18.04.: Debugging Humidity calibration
;         Now you can override either Wet or dry calibration routing so to
;         preserve the old value
; 18.04.: Moved some string definitions to pgm page 1 to free space in pgm Page 2
; 28.04.: Modified hum measurement. Included tolerance up to 110% humidity to compensate
;         small calibration errors. In this case, error flag is set nevertheless.

;todo:
;keine manuelle ventilbedienung wenn pgm=running ??? ntig????
; expanded Error control &Error Definition
;
; ToDo: Display error sometimes when handling menu (stack overflow)
;temporary solution: no display update out of isr to limit stack usage
;
; Used Controller: PIC 16F690
; Pins & Ports:
; RC0..RC3: 	LCD-Display Bus (4Bit)
; RC4:			LCD-Display E
; RC5:			LCD-Display RS; R/W wired to low (all write)
; RB4..RB6		keys
; RB7			LED
; RC6			SCL (I2C)
; RC7			SDA (I2C)
; RA4			Sensor Waterlevel
; RA5			Sensor Humidity
; RA2			Backlight LCD Display
;
;Errorflags (displayed as Hex)
;Bit 0: Humidity Sensor Error. Value out of range, check calibration
;Bit 1: Calibration Error. dry-value lower than wet value
;Bit 2: Configuration in eeprom is not valid
;
; see schematic for more Details
;
; Fosc=4MHz internal
	
	LIST   P=PIC16F690

	#include <p16f690.inc> 

    errorlevel  -302		;No bank select warnings
    errorlevel  -306		;No page select warnings

	__config (_INTOSCIO & _WDT_OFF & _PWRTE_OFF & _MCLRE_OFF & _CP_OFF & _BOD_OFF & _IESO_OFF & _FCMEN_OFF) 
;----------------------------------------------------------------
;variables
w_tmp		EQU	0x20    ; used for context saving Register "W"
status_tmp	EQU	0x21    ; used for context saving Register "Status"
pclath_tmp	equ	0x22	; used for context saving Register "PCLATH"
pclath_save	equ 0x23	;used to save pclath within computed gotos
	cblock 0x24
	tmp_tmp				;context saving
	tmp1_tmp
	f0_tmp				;dto
	d1				;Delay-Variable 1	
	d2				;Delay-Variable 2
	d3				;Delay-Variable 3
	count			;count register for I2C routines
	tmp				;tmporary register
	tmp1
	temp			;used for 16 bit ascii transformation
	
	tmpGIE			;temp register for saving GIE state in routines

	f0				;variables for arithmetic operations
	f1
	f2
	f3
	y
	counter
	Hi				;used for 16bit ascii transformation and tmr1 operations
	Lo
	
	offset			;used for table read

	sec
	min
	hr				;current time in bcd format
	
	tmphr
	tmpmin			;tmp reg used for timesetting (within menu)
	
	starthr
	startmin		;starttime in hex
	
	dur1
	dur2
	dur3
	dur4			;duration of valves
	tmpdur			;temp register duration (used in Menu)
	currdur			;current duration (used in menu)
	pgmphase			;current pgm-phase (0..4)
	phasedur			;current duration of phase
	
	refdur			;duration of refilling
	refint			;refill check interval
	
	hum				;humidity
	humtres			;humidity treshold
	humint			;humidity interval
	cal1h
	cal1l	
    cal2h
    cal2l
	caldelta		;sensor calibration value
	
	;----Stati (means live status of relais
	stat1			;status 1..4
	stat2			;bit0: on/off
	stat3			;bit1: auto/man
	stat4			;
	statp			;status Pump (only auto/x or man/on
	statn			;Status Refill (Refillsystem, only on/off)
	
	statusexp		;Status of Port expander PCF8574
	
	tickcounter		;counts Timer0 interrupts (16 ticks=1sec)
	seccounter		;counts secs (60=1min)
	
	senstimer		;counter of minutes between sensor-checks
	refilltimer		;same for refill system
	refilldur		;refill duration counter
	keytimer		;counter of secs used for keychecking
	bltimer			;backlighttimer
	
	flags			;holds diverse flags (see #defines)
	flags1
	errflags

	Dispdata		;holds Dislay Data
	keydata			;status of keys

	look_hi			;high and lowbyte for stringpointer
	look_lo

	asciiO			;ASCII Ones
	asciiT			;ASCII Tens
	asciiH			;ASCII Hundreds
	asciiK			;ASCII Thousands (kilo)
	asciiTK			;ASCII Ten thousands (ten kilo)
	eprom_adr		;eeprom Adress tmp
	eprom_dat		;eeprom data tmp
	
	endc
;----------------------------------------------------------------
;constants
DISPPort	equ	PORTC	;Port Register for LCD-Display
#define DISPRS	PORTC,5	;RS of LCD
#define DISPE	PORTC,4	;E of LCD

#define LED		PORTB,7	;LED
#define Backlight	PORTA,2	;LCD Backlight
#define Water	PORTA,4	;Water Level sensor

#define SDA		PORTC,7	;SDA
#define SCL		PORTC,6	;SCL
#define SDA_T	TRISC,7	;Tris-Bit for SDA
#define SCL_T	TRISC,6	;Tris-Bit for SCL
;flags
#define dispupd		flags,0	;flag for display-Update
#define refilluse	flags,1	;flag for refill system on/off
#define pgmrun		flags,2	;flag for pgm running
#define pgmuse		flags,3 ;flag for pgm on/off
#define	humuse		flags,4	;flag for humidity sensor on/off
#define	confchange	flags,5	;flag if config has changed
#define	manstartstop	flags,6	;flag if manual start or stop has been pressed
#define	refillrun	flags,7	;flag for refill-system running
#define	humrun	flags1,0	;flag for humidity check

#define	humerr	errflags,0	;flag if humidity-Sensor-Error
#define caliberr	errflags,1	;flag used in sensor calibration 
#define	configerr	errflags,2	;flag used if no valid config loaded
								;set if high-val < low-val

;----------------------------------------------------------------
;macro definitions
;----------------------------------------------------------------
;strout Macro to output string at current cursor position
strout	macro	stringaddr		
		movlw	high stringaddr	;save high byte 
		movwf	look_hi			
		movlw	low stringaddr	;save low byte
		movwf	look_lo
		call	Disp_string		;output it
		endm

Lstrout	macro	stringaddr		;long call version 
		movlw	high stringaddr	;save high byte 
		movwf	look_hi			
		movlw	low stringaddr	;save low byte
		movwf	look_lo
		movlw	high Disp_string
		movwf	PCLATH
		call	Disp_string		;output it
		clrf	PCLATH
		endm

;set to Register Bank 0
SET_BANK0 MACRO
	bcf STATUS, RP0
	bcf STATUS, RP1
	ENDM

;set to Register Bank 1
SET_BANK1 MACRO
	bsf	STATUS, RP0
	bcf	STATUS, RP1
	ENDM
;set to Register Bank 2
SET_BANK2 MACRO
	bcf	STATUS, RP0
	bsf	STATUS, RP1
	ENDM
; **********************************************************************
; **********************************************************************
	org 0 
	goto INIT
	ORG     0x004           ; interrupt vector location
	movwf   w_tmp          ; save off current W register contents
	swapf   STATUS,W        ; move status register into W register
	clrf    STATUS          ; select Bank 0
	movwf   status_tmp     ; save off contents of STATUS register
	movf	PCLATH,w
	movwf	pclath_tmp
	clrf	PCLATH
					
	movf	f0,w
	movwf	f0_tmp
	movf	tmp,w
	movwf	tmp_tmp
	movf	tmp1,w
	movwf	tmp1_tmp
	
	btfsc   INTCON,T0IF     ; If Timer 0 caused the interrupt, handle it.
	goto    Timer0_ISR		;no call because wo don't have any stack level left
tmr0return

	movf	f0_tmp,w
	movwf	f0
	movf	tmp_tmp,w
	movwf	tmp
	movf	tmp1_tmp,w
	movwf	tmp1
	
	movf	pclath_tmp,w
	movwf	PCLATH
	swapf   status_tmp,w     ; retrieve copy of STATUS register
	movwf   STATUS            ; restore pre-isr STATUS register contents
	swapf   w_tmp,f
	swapf   w_tmp,w          ; restore pre-isr W register contents
	retfie                    ; return from interrupt
	
; *********************************************
; Timer 0 Interrupt handler.
; Timer 0 has overflowed
; *********************************************
Timer0_ISR
	movlw	0xff
	incf	keytimer,f		;increment keytimer
	skpnz
	movwf	keytimer		;if overflow then set to max
	incf	tickcounter,f	;tickcounter+1
	movf	tickcounter,w
	xorlw	0x10			;tickcounter 16?
	btfss	STATUS,Z		;if yes: 1 sec over
	goto	tmr0end			;if not set then end isr
	;----- 1 second over
	;movlw	080h			;blink LED
	;xorwf	PORTB,f
	incf	bltimer,f
	skpnz
	bcf		Backlight		;clear Backlight after 255 sec (4min15sec)
	clrf	tickcounter
	call	handle_refill	;we need to check refill system every sec..
	incf	seccounter,f	;seccounter+1
	movf	seccounter,w
	xorlw	d'60'			;xor with 60 -> counter already 60?
	btfss	STATUS,Z
	goto	tmr0end		; if minute not over then end isr
	;----- 1 minute over------------------------
	incf	senstimer,f
	incf	refilltimer,f
	incf	refilldur,f
	btfsc	pgmrun		;if program is running
	incf	phasedur,f		;...then increment duration of current phase
	
	;-----handle time	
	call	rtc_brst_rd
	movf	sec,w			;synchronize seccounter with rtc
	movwf	f0
	call	bcd2bin
	movwf	seccounter
	;call	disp_time		;+++++ todo: Stack overflow correct this later
	bsf		dispupd		; set flag Display update

	;-----check hum sensor start condition
	movf	senstimer,w		;check if senstimer is over
	xorwf	humint,w		
	skpz				;skip if equal = time arrived
	goto	tmr_0_0_end		;not  yet, so dont care
	clrf	senstimer		;reset senstimer
	;btfsc	humuse			;only set humrun if hum usage is on
	bsf		humrun		;set flag
tmr_0_0_end

	;-----check program-start condition
	btfss	pgmuse			;check if we shall use the program
	goto	tmr0_1_end			;no, then don't mind
	movf	hr,w			;otherwise check if starttime arrived
	movwf	f0
	call	bcd2bin
	xorwf	starthr,w
	skpz
	goto	tmr0_1_end
	movf	min,w
	movwf	f0
	call	bcd2bin
	xorwf	startmin,w
	skpz
	goto	tmr0_1_end		;not yet starttime
	;we get here if time=starttime
	bsf		pgmrun			;set pgm to run-state
							;but check humidity....
	btfss	humuse			;only if humidity on
	goto	tmr0_1			;if no humidity sensor then dont check humidity
	movf	hum,w
	subwf	humtres,w
	skpc
	bcf		pgmrun			;clear pgm run if humindity > hum treshold
tmr0_1
	;diplay update (run)
	;call	disp_msg		;+++++Stack overflow correct this later
	bsf		dispupd		;set flag Display update
tmr0_1_end
	call	handle_pgm

	;-----check refill-start condition
	btfss	refilluse		;check if we shall use the refill system
	goto	tmr0_2_end		;no, then don't mind
	movf	refilltimer,w	;check if refilltimer over
	xorwf	refint,w		
	skpz
	goto	tmr0_2_end		;no, then end check
	;else...
	clrf	refilltimer		;reset timer
	clrf	refilldur		;reset refill duration
	;---check water level
	bsf		statusexp,7		;power to water sensor on
	movf	statusexp,w
	call	expander_wr
	call	LDelay10ms		;wait to stabilize
	btfsc	Water			;check water level
	bsf		refillrun		;high=water level low so refill on
	btfsc	refillrun
	bsf		statn,0		;set status for display
	btfsc	refillrun
	bsf		dispupd		;set display to update
	bcf		statusexp,7		;power to water sensor off
	movf	statusexp,w
	call	expander_wr	
	call	handle_refill	;call handle refill for the 1st time, 
							;next loop from top of isr
tmr0_2_end

tmr0end
	BCF 	INTCON,T0IF		; Clear Timer0 flag and continue.
    goto	tmr0return		;hard coded return to save one stack level
;---------------------------------------------------  
;-----handle refill system (called by isr)
handle_refill
	btfss	refillrun		;refill system running?
	goto	href_end		;no, then end routine
	movf	refilldur,w
	xorwf	refdur,w
	skpz
	goto	href_1
	;---end refilling because maxduration is over
	bcf		refillrun
	bcf		statn,0	
	bsf		dispupd			;displayupdate
	clrf	refilltimer
	bcf		statusexp,4		;refill valve off
	;special treatment for pump
	btfsc	pgmrun				;program running?
	goto	href_0_1			;yes,then don't touch pump
	btfsc	statp,1			;manual mode?
	bcf		statp,0			;no, then set status=off
	btfsc	statp,1			;manual mode?
	bsf		statusexp,5		;no, then pump off 
href_0_1
	movf	statusexp,w
	call	expander_wr	
	goto	href_end
href_1
	;---we get here if refill is running and refill duration not over
	bsf		statusexp,4		;refill valve on
	;---special treatment for pump not needed -if mode is man it already is on
	;---if pgmrun then it is on also
	bcf		statusexp,5		;pump on
	bsf		statp,0
	;check water level!!!!
	bsf		statusexp,7		;power to water sensor on
	movf	statusexp,w
	call	expander_wr
	call	LDelay10ms		;wait to stabilize
	btfsc	Water			;check water level
	goto	href_1_1		;high=water level low
	bcf		refillrun		;low=water level high so refill off
	bcf		statn,0	
	bsf		dispupd			;displayupdate
	clrf	refilltimer
	bcf		statusexp,4		;refill valve off
	;---special treatment for pump
	btfsc	pgmrun				;program running?
	goto	href_1_1			;yes,then don't touch pump
	btfsc	statp,1			;manual mode?
	bcf		statp,0			;no, then set status=off
	btfsc	statp,1			;manual mode?
	bsf		statusexp,5		;no, then pump off
href_1_1
	bcf		statusexp,7		;power to water sensor off
	movf	statusexp,w
	call	expander_wr	
href_end
	return

;-----handle program logic (called by isr)
handle_pgm
	;btfsc	manstartstop
	bcf		manstartstop
	btfss	pgmrun		;program running?
	goto	hpgm_2_end		;no, then finish   +++todo: what happens if run=0??
	bcf		statusexp,5		;pump on
	bsf		statp,0			;status=on
	movf	pgmphase,w
	xorlw	0x00
	skpz
	goto	hpgm_2_0
	clrf	phasedur
	movlw	0x01			;if phase  was 0 then set to 1 (start)
	movwf	pgmphase
hpgm_2_0
	xorlw	0x01			;phase 1?
	skpnz
	goto	hpgm_2_1		;yes, then handle phase 1
	xorlw	0x02^0x01		;phase 2?
	skpnz
	goto	hpgm_2_2		;yes, then handle phase 2
	xorlw	0x03^0x02		;phase 3?
	skpnz
	goto	hpgm_2_3		;yes, then handle phase 3
	xorlw	0x04^0x03		;phase 4?
	skpnz
	goto	hpgm_2_4		;yes, then handle phase 4
	;we get here if pgm is over (phase=5)

	goto	hpgm_2_end

hpgm_2_1				;handle phase 1....
	movf	stat1,w		;status =0 (man off)?
	skpz
	goto	hpgm_2_1a		;no, then go on
	movlw	0x02
	movwf	pgmphase		;set new phase
	clrf	phasedur
	goto	hpgm_2_2		;goto next phase
hpgm_2_1a
	bsf		statusexp,0		;set relais 1 on
	movlw	0x03		;set status-byte 
	movwf	stat1
	movf	dur1,w		;check if phase 1 is over
	xorwf	phasedur,w
	skpz
	goto	hpgm_2_end		;if phase is not over then end
	bcf		statusexp,0		;clear relais 1
	bcf		stat1,0		;clear status-byte
	movlw	0x02
	movwf	pgmphase		;set new phase
	clrf	phasedur
hpgm_2_2				;handle phase 2....
	movf	stat2,w		;status =0 (man off)?
	skpz
	goto	hpgm_2_2a		;no, then go on
	movlw	0x03
	movwf	pgmphase		;set new phase
	clrf	phasedur
	goto	hpgm_2_3		;goto next phase
hpgm_2_2a
	bsf		statusexp,1		;set relais 2 on
	movlw	0x03		;set status-byte 
	movwf	stat2
	movf	dur2,w		;check if phase 2 is over
	xorwf	phasedur,w
	skpz
	goto	hpgm_2_end		;if phase is not over then end
	bcf		statusexp,1		;clear relais 2
	bcf		stat2,0		;clear status-byte
	movlw	0x03
	movwf	pgmphase		;set new phase
	clrf	phasedur
hpgm_2_3				;handle phase 3....
	movf	stat3,w		;status =0 (man off)?
	skpz
	goto	hpgm_2_3a		;no, then go on
	movlw	0x04
	movwf	pgmphase		;set new phase
	clrf	phasedur
	goto	hpgm_2_4		;goto next phase
hpgm_2_3a
	bsf		statusexp,2		;set relais 3 on
	movlw	0x03		;set status-byte 
	movwf	stat3
	movf	dur3,w		;check if phase 3 is over
	xorwf	phasedur,w
	skpz
	goto	hpgm_2_end		;if phase is not over then end
	bcf		statusexp,2		;clear relais 3
	bcf		stat3,0		;clear status-byte
	movlw	0x04
	movwf	pgmphase		;set new phase
	clrf	phasedur
hpgm_2_4				;handle phase 4....
	movf	stat4,w		;status =0 (man off)?
	skpz
	goto	hpgm_2_4a		;no, then go on
	movlw	0x05
	movwf	pgmphase		;set new phase
	clrf	phasedur
	goto	hpgm_2_end		;goto next phase (which is end)
hpgm_2_4a
	bsf		statusexp,3		;set relais 4 on
	movlw	0x03		;set status-byte 
	movwf	stat4
	movf	dur4,w		;check if phase 4 is over
	xorwf	phasedur,w
	skpz
	goto	hpgm_2_end		;if phase is not over then end
	bcf		statusexp,3		;clear relais 1
	bcf		stat4,0		;clear status-byte
	movlw	0x05
	movwf	pgmphase		;set new phase
	clrf	phasedur
hpgm_2_end

	movf	pgmphase,w
	xorlw	0x05			;already Phase 5?
	skpz
	goto	hpgm_3			;no, then end
	clrf	pgmphase		;else clean up
	bcf		pgmrun
	btfsc	statp,1			;manual mode?
	bcf		statp,0			;no, then set status=off
	btfsc	statp,1			;manual mode?
	bsf		statusexp,5		;no, then pump off
	;reset relais status, reset status stat1..4
	bcf		statusexp,0
	bcf		statusexp,1
	bcf		statusexp,2
	bcf		statusexp,3
	bcf		stat1,0
	bcf		stat2,0
	bcf		stat3,0
	bcf		stat4,0
hpgm_3
	movf	statusexp,w
	call	expander_wr
	bsf		dispupd
	return

;------------------------------------------------------------------
;initializes Processor
;------------------------------------------------------------------
INIT
	SET_BANK1	 		;select Register Page 1 
	bsf		OSCCON,IRCF2	;select 4MHz
	bsf		OSCCON,IRCF1
	bcf		OSCCON,IRCF0
	;configure input/output
	bcf 	TRISC,0		;LCD Bit0..3 output
	bcf 	TRISC,1		;
	bcf 	TRISC,2		;
	bcf 	TRISC,3		;
	bcf 	TRISC,4		;LCD E output
	bcf 	TRISC,5		;LCD RS output
	bsf 	TRISC,6		;SCL input
	bsf 	TRISC,7		;SDA input
	
	bcf		TRISA,2		;LCD Backlight output
	bsf		TRISA,4		;Waterlevel input
	bsf		TRISA,5		;Sensor1 / T1clk input

	bsf		TRISB,4		;keys, all input
	bsf		TRISB,5
	bsf		TRISB,6
	bcf		TRISB,7		;LED = output
	SET_BANK2
	;configure analog I/O
	bsf		WPUB,WPUB4	;weak pull up keys
	bsf		WPUB,WPUB5	;
	bsf		WPUB,WPUB6	;
	clrf 	ANSEL		;all digital I/O
	clrf	ANSELH
	SET_BANK0
	
	;initialise Timer 0
	; 
	;we are in Bank 0...
	clrwdt				;Routine from Datasheet Microchip
	clrf	TMR0	
	bsf		STATUS, RP0	;Switch to BANK 1
	movlw	b'01010111'		;Pullup enable, Prescaler to TMR0, Prescaler 1:256
	movwf OPTION_REG
	SET_BANK0

	;initialise Timer 1 as counter
	movlw	b'00000111'	;activate TMR1: No Gate control, 1:1, LP off;
						;Not synchronized, external Clock, Enabled
	movwf	T1CON
	clrf	TMR1L
	clrf 	TMR1H


	;----now set reasonable values to variables
	clrf	flags
	clrf	flags1
	clrf	errflags
	clrf	tickcounter
	clrf	senstimer
	clrf	bltimer

	clrf	keytimer
		
	clrf	PORTC		;port C all to 0 (init)
	bsf		SDA
	bsf		SCL			;SCL and SDA high
	bcf		LED			;LED off
	bsf		Backlight	;Backlight on
	
	movlw	0x02		;valves 1..4 auto, off
	movwf	statp		;Pump auto, off
	clrf	statn		;Refill off
	;----Program
	bcf		pgmrun
	bcf		manstartstop
	clrf	pgmphase		;reset pgmphase to 0
	clrf	phasedur		;current duration of phase =0
	;----humidity
	clrf	hum			;humidity 0%
	bsf		humrun		;we want an initial measurement at start
	;----refill system
	bcf		refillrun
	clrf	refilltimer
	clrf	refilldur

	call	load_prefs
	
	
	;-----Port expander default state
	call	i2c_stop
	movlw	b'00100000'
	movwf	statusexp	;save
	call	expander_wr	;output to I2C Port expander (all off)
	
	;-----Display init and welcome message
	bsf		dispupd
	movlw	high Disp_Init	;all on Page 1
	movwf	PCLATH
	call	Disp_Init	;Init Display
	call	Disp_clear	;clear Display
	call	Delay10ms	;wait for Display
	
	movlw	b'11000000'		;cursor on pos 1 at 2nd row
	call	Disp_send_conf
	strout	msg_welcome		;start-message on display
	movlw	b'10010000'		;cursor on pos 1 at 3nd row
	call	Disp_send_conf
	strout	msg_welcome1
	clrf	PCLATH		;back to page 0	
	
	movlw	d'255'			;wait ca. 2,5s (for user to read it)
	movwf	d3
	decfsz	d3,f
	goto	$+2
	goto	$+3
	call	LDelay10ms
	goto	$-4
	;-----
	;--now get current time 
	call	i2c_stop
	call	rtc_brst_rd		;get time
	;call	rtc_brst_wr		;save to make sure 24 mode & OSC is running
	call	rtc_init		;to set control reg
	movf	sec,w			;synchronize seccounter with rtc
	movwf	f0
	call	bcd2bin
	movwf	seccounter
	;---
	clrf 	TMR0
	clrf	INTCON			;initialize Interrupts
	bsf		INTCON,T0IE		;enable Timer0 interrupts
	bsf 	INTCON,GIE		;enable global interrupts
							;now timer 0 fires
							
							

;#############################################################################
;here to start main routine###################################################
;#############################################################################
main
	movlw	high Disp_Init	;all on Page 1
	movwf	PCLATH
	call	Disp_clear		;clear display
	call	Delay10ms		;needs time to finish...
	clrf	PCLATH
	
mainloop
	btfss	humrun
	goto	mainloop0_0
	call	measure_hum
	bsf		dispupd
	bcf		humrun
mainloop0_0
	movf	errflags,w
	skpnz
	bcf		LED
	skpz
	bsf		LED				;if some errror exists then set error Indicator

	btfss	dispupd
	goto	mainloop0
	bcf		dispupd
	call	disp_on			;switch Display on  (experimental)
	call	disp_time		;update Display
	call	disp_hum
	call	disp_msg
	call	disp_stats
mainloop0
	btfsc	manstartstop	;manual start or stop pressed?
	call	handle_pgm		;yes, then handle pgm
	btfsc	confchange		;config changed during key/menu handling?
	call	save_prefs		;yes, then save config
	bcf		confchange		;and reset config change flag

	call	LDelay100ms		;and wait ...
	call	Lkeycheck		;returns status of buttons in w
	andlw	0xff			;to reset Status of Z
	skpnz					;if not 0 then handle menu
	goto	mainloop		;otherwise do nothing
	bsf		Backlight		;key pressed, Backlight on
	clrf	bltimer
	call	Lgetkey			;get new status
	xorlw	0x01			;Menu pressed?
	skpnz					
	goto	mainloop1		;yes, mainmenu
	;movf	keydata,w
	xorlw	0x02^0x01
	skpnz
	goto	mainloop2		;yes, pump changed
	;movf	keydata,w		;if we get here then 3..4 was pressed
	xorlw	0x03^0x02
	skpnz
	goto	mainloop3		;yes, valve4 changed
	;movf	keydata,w
	xorlw	0x04^0x03
	skpnz
	goto	mainloop4		;yes, valve3 changed
	;movf	keydata,w
	xorlw	0x05^0x04
	skpnz
	goto	mainloop5		;yes, valve2 changed
	;movf	keydata,w
	xorlw	0x06^0x05
	skpnz
	goto	mainloop6		;yes, valve1 changed
	call	LDelay100ms		;to insure no keybounce anymore
	call	Lwaitforrelease

	goto	mainloop		;goto mainmenu
;---------
mainloop1
	movlw	high handle_mainmenu
	movwf	PCLATH
	call	handle_mainmenu
	clrf	PCLATH
	call	LDelay100ms		;to insure no keybounce anymore
	call	Lwaitforrelease
	bsf		dispupd
	goto	main				;to assure right text on display

mainloop2					;pump pressed
	btfss	statp,0			;check pump status
	goto	mainloop21		;if off then jump
	movlw	0x02
	movwf	statp			;set pump status off, auto
	bsf		statusexp,5		;set status on statusbyte
	movf	statusexp,w
	call	expander_wr		;output to i2c
	goto	mainloop22
mainloop21
	movlw	0x01
	movwf	statp			;set pump status on, man
	bcf		statusexp,5		;set status on statusbyte
	movf	statusexp,w
	call	expander_wr		;output to i2c
mainloop22
	call	disp_stats		;update Display
	call	LDelay100ms		;to insure no keybounce anymore
	call	Lwaitforrelease
	;experimental
	;call	measure_hum
	;bsf		dispupd
	;experimetal end
	goto	mainloop		;display already updated
;key 3=Valve4 pressed
mainloop3
	clrf	keytimer		;to measure how long key is pressed
	movf	stat4,w
	xorlw	0x01			;was man on 
	skpnz
	goto	mainloop31
	xorlw	0x02^0x01		;was auto off
	skpnz
	goto	mainloop32
	xorlw	0x03^0x02		;was auto on
	skpnz
	goto	mainloop31
	goto	mainloop33
	
mainloop31
	movlw	0x02			;set auto off
	movwf	stat4
	goto	mainloop33
mainloop32
	movlw	0x01			;set man on
	movwf	stat4
mainloop33
	call	disp_stats		;update Display
	call	Lkeycheck
	andlw	0xff			;to reset status of z
	skpnz
	goto	mainloop35		;if released then go on
	movf	keytimer,w
	xorlw	d'48'			;3 sec
	skpz
	goto	mainloop33+1	;not over, then keep checking, no display update
	;we get here if 3secs+ pressed
	movf	stat4,w			;what was state before?
	skpnz
	goto	mainloop34		;it was 0, jump
	clrf	stat4			;it wasnt 0, set to man off
	bsf		confchange
	goto	mainloop35
	
mainloop34
	movlw	0x02			;auto off
	movwf	stat4
	bsf		confchange
mainloop35
	bcf		statusexp,3		;clear output state
	btfsc	stat4,0	
	bsf		statusexp,3		;if status=on then set output on
	
	call	disp_stats		;update Display
	call	LDelay100ms		;to insure no keybounce anymore
	call	Lwaitforrelease
	
	movf	statusexp,w
	call	expander_wr		;output to i2c
	goto	mainloop		;display already updated
;key 4=Valve3 pressed
mainloop4
	clrf	keytimer		;to measure how long key is pressed
	movf	stat3,w
	xorlw	0x01			;was man on 
	skpnz
	goto	mainloop41
	xorlw	0x02^0x01		;was auto off
	skpnz
	goto	mainloop42
	xorlw	0x03^0x02		;was auto on
	skpnz
	goto	mainloop41
	goto	mainloop43
	
mainloop41
	movlw	0x02			;set auto off
	movwf	stat3
	goto	mainloop43
mainloop42
	movlw	0x01			;set man on
	movwf	stat3
mainloop43
	call	disp_stats		;update Display
	call	Lkeycheck
	andlw	0xff			;to reset status of z
	skpnz
	goto	mainloop45		;if released then go on
	movf	keytimer,w
	xorlw	d'48'			;3 sec
	skpz
	goto	mainloop43+1	;not over, then keep checking, no display update
	;we get here if 3secs+ pressed
	movf	stat3,w			;what was state before?
	skpnz
	goto	mainloop44		;it was 0, jump
	clrf	stat3			;it wasnt 0, set to man off
	bsf	confchange
	goto	mainloop45
mainloop44
	movlw	0x02			;auto off
	movwf	stat3
	bsf		confchange
mainloop45
	bcf		statusexp,2		;clear output state
	btfsc	stat3,0	
	bsf		statusexp,2		;if status=on then set output on
	
	call	disp_stats		;update Display
	call	LDelay100ms		;to insure no keybounce anymore
	call	Lwaitforrelease
	
	movf	statusexp,w
	call	expander_wr		;output to i2c
	goto	mainloop		;display already updated
;key 5=Valve2 pressed
mainloop5
	clrf	keytimer		;to measure how long key is pressed
	movf	stat2,w
	xorlw	0x01			;was man on 
	skpnz
	goto	mainloop51
	xorlw	0x02^0x01		;was auto off
	skpnz
	goto	mainloop52
	xorlw	0x03^0x02		;was auto on
	skpnz
	goto	mainloop51
	goto	mainloop53
	
mainloop51
	movlw	0x02			;set auto off
	movwf	stat2
	goto	mainloop53
mainloop52
	movlw	0x01			;set man on
	movwf	stat2
mainloop53
	call	disp_stats		;update Display
	call	Lkeycheck
	andlw	0xff			;to reset status of z
	skpnz
	goto	mainloop55		;if released then go on
	movf	keytimer,w
	xorlw	d'48'			;3 sec
	skpz
	goto	mainloop53+1	;not over, then keep checking, no display update
	;we get here if 3secs+ pressed
	movf	stat2,w			;what was state before?
	skpnz
	goto	mainloop54		;it was 0, jump
	clrf	stat2			;it wasnt 0, set to man off
	bsf	confchange
	goto	mainloop55
mainloop54
	movlw	0x02			;auto off
	movwf	stat2
	bsf		confchange
mainloop55
	bcf		statusexp,1		;clear output state
	btfsc	stat2,0	
	bsf		statusexp,1		;if status=on then set output on
	
	call	disp_stats		;update Display
	call	LDelay100ms		;to insure no keybounce anymore
	call	Lwaitforrelease
	
	movf	statusexp,w
	call	expander_wr		;output to i2c
	goto	mainloop		;display already updated
;key 6=Valve1 pressed
mainloop6
	clrf	keytimer		;to measure how long key is pressed
	movf	stat1,w
	xorlw	0x01			;was man on 
	skpnz
	goto	mainloop61
	xorlw	0x02^0x01		;was auto off
	skpnz
	goto	mainloop62
	xorlw	0x03^0x02		;was auto on
	skpnz
	goto	mainloop61
	goto	mainloop63
	
mainloop61
	movlw	0x02			;set auto off
	movwf	stat1
	goto	mainloop63
mainloop62
	movlw	0x01			;set man on
	movwf	stat1
mainloop63
	call	disp_stats		;update Display
	call	Lkeycheck
	andlw	0xff			;to reset status of z
	skpnz
	goto	mainloop65		;if released then go on
	movf	keytimer,w
	xorlw	d'48'			;3 sec
	skpz
	goto	mainloop63+1	;not over, then keep checking, no display update
	;we get here if 3secs+ pressed
	movf	stat1,w			;what was state before?
	skpnz
	goto	mainloop64		;it was 0, jump
	clrf	stat1			;it wasnt 0, set to man off
	bsf	confchange
	goto	mainloop65
mainloop64
	movlw	0x02			;auto off
	movwf	stat1
	bsf		confchange
mainloop65
	bcf		statusexp,0		;clear output state
	btfsc	stat1,0	
	bsf		statusexp,0		;if status=on then set output on
	
	call	disp_stats		;update Display
	call	LDelay100ms		;to insure no keybounce anymore
	call	Lwaitforrelease
	movf	statusexp,w
	call	expander_wr		;output to i2c
	goto	mainloop		;display already updated
;---------------------------------------------------------------
;load Prefs
;loads Preferences from internal eeprom if valid config is saved
; in eeprom. Set to default vaalues ottherwise
;---------------------------------------------------------------
load_prefs
	clrf	eprom_adr		;read pos 0 =valid-check
	call	eprom_read
	xorlw	0x55
	skpz					;go on if confg is valid
	goto	load_prefs_def	;set to defaults otherwise
	;config is valid, now load them
	;stati
	call	read_it
	movwf	stat1
	call	read_it
	movwf	stat2
	call	read_it
	movwf	stat3
	call	read_it
	movwf	stat4
	;humidity
	bcf		humuse			;set sensor off
	call	read_it
	skpz					;skip next if value=0(off)
	bsf		humuse			;set on otherwise
	call	read_it
	movwf	cal1h
	call	read_it
	movwf	cal1l
	call	read_it
	movwf	caldelta
	call	read_it
	movwf	humtres
	call	read_it
	movwf	humint
	;refill
	bcf		refilluse		;set sensor off
	call	read_it
	skpz					;skip next if value=0(off)
	bsf		refilluse		;set on otherwise
	call	read_it
	movwf	refint
	call	read_it
	movwf	refdur
	;program
	bcf		pgmuse			;set sensor off
	call	read_it
	skpz					;skip next if value=0(off)
	bsf		pgmuse			;set on otherwise
	call	read_it
	movwf	starthr
	call	read_it
	movwf	startmin
	call	read_it
	movwf	dur1
	call	read_it
	movwf	dur2
	call	read_it
	movwf	dur3
	call	read_it
	movwf	dur4
    ;new prefs added 4/2014
	call	read_it
	movwf	cal2h
	call	read_it
	movwf	cal2l
	goto	load_prefs_end
	
load_prefs_def			;set default to prefs
	bsf		configerr	;if we got here, config in eeprom is not valid
	;stati
	movlw	0x02		;valves 1..4 auto, off
	movwf	stat1
	movwf	stat2
	movwf	stat3
	movwf	stat4
	;humidity
	bcf		humuse	;don't use hum sensor
	movlw	d'50'
	movwf	humtres		;treshold 50%
	movlw	d'10'
	movwf	humint		;interval 10min
	movlw	0x01
	movwf	caldelta
	;refill
	bcf		refilluse	;don't use refill system
	movlw	d'60'
	movwf	refint
	movlw	d'1'
	movwf	refdur
	;program
	bcf		pgmuse	;don't use program system
	clrf	starthr		;starttime: 00:00 (doesn't matter)
	clrf	startmin
	clrf	dur1			;all durations 0min
	clrf	dur2
	clrf	dur3
	clrf	dur4

	goto	load_prefs_end
read_it
	incf	eprom_adr,f		;next
	call	eprom_read
	return
load_prefs_end	
	return
;---------------------------------------------------------------
;save_Prefs
; save Preferences to internal eeprom
;---------------------------------------------------------------
save_prefs
	clrf	eprom_adr		;begin saving at pos 0
	movlw	0x55			;valid config marker
	movwf	eprom_dat
	call	eprom_write
	;stati
	movf	stat1,w
	andlw	0x02			;only save auto/man
	call	save_it
	movf	stat2,w
	andlw	0x02			;only save auto/man
	call	save_it
	movf	stat3,w
	andlw	0x02			;only save auto/man
	call	save_it
	movf	stat4,w
	andlw	0x02			;only save auto/man
	call	save_it
	;humidity
	clrw				;initially clear w =sensor off
	btfsc	humuse		;if flag=on
	movlw	0x01		;then set config on
	call	save_it
	movf	cal1h,w
	call	save_it	
	movf	cal1l,w
	call	save_it	
	movf	caldelta,w
	call	save_it	
	movf	humtres,w
	call	save_it	
	movf	humint,w
	call	save_it
	;refill
	clrw				;initially clear w =refill system off
	btfsc	refilluse	;if flag=on
	movlw	0x01		;then set config on
	call	save_it
	movf	refint,w
	call	save_it	
	movf	refdur,w
	call	save_it	
	;program
	clrw				;initially clear w =program off
	btfsc	pgmuse		;if flag=on
	movlw	0x01		;then set config on
	call	save_it
	movf	starthr,w
	call	save_it
	movf	startmin,w
	call	save_it
	movf	dur1,w
	call	save_it
	movf	dur2,w
	call	save_it
	movf	dur3,w
	call	save_it
	movf	dur4,w
	call	save_it
    ;new prefs added 4/2014
	movf	cal2h,w
	call	save_it
	movf	cal2l,w
	call	save_it
	goto	save_prefs_end
save_it
	movwf	eprom_dat
	incf	eprom_adr,f
	call	eprom_write
	return
save_prefs_end
	return
;---------------------------------------------------------------	
;Display Update Routines
;---------------------------------------------------------------
disp_on					;displays time on line 1
	movlw	b'00101000'		;4bits, 2 line, 5x7 dots
	call 	LDisp_send_conf
	call 	LDelay10ms
	movlw	b'00001100'		;Display on, Cursor off, No Blinking
	call 	LDisp_send_conf
	call 	LDelay10ms
	return
;----
disp_time					;displays time on line 1
	movlw	b'10000000'		;cursor at pos 1 at 1st line
	call	LDisp_send_conf
	movf	hr,w
	movwf	f0
	call 	bcd2bin
	call	Lbin2asc
	movf	asciiT,w		;
	call	LDisp_send_data
	movf	asciiO,w		;
	call	LDisp_send_data
	movlw	':'
	call	LDisp_send_data
	movf	min,w
	movwf	f0
	call 	bcd2bin
	call	Lbin2asc
	movf	asciiT,w		;
	call	LDisp_send_data
	movf	asciiO,w		;
	call	LDisp_send_data
	return
;----
disp_hum					;display humidity on line 1
	movlw	b'10001100'		;cursor at pos 13 at 1st line	
	call	LDisp_send_conf
	movf	hum,w
	call	Lbin2asc
	movf	asciiH,w		;
	call	LDisp_send_data
	movf	asciiT,w		;
	call	LDisp_send_data
	movf	asciiO,w		;
	call	LDisp_send_data
	movlw	'%'
	call	LDisp_send_data
	return
;----
disp_msg					;display message on line 1
	movlw	b'10000111'		;cursor at pos 7 at 1st line	
	call	LDisp_send_conf
	btfss	pgmrun			;pgm running
	goto	disp_msg_1
	Lstrout	msg_run
	goto	disp_msg_end
disp_msg_1
	movf	errflags,w
	skpnz
	goto	disp_msg_2
	movlw	a'E'
	call	LDisp_send_data
	;high nibble of error
	swapf	errflags,w
	andlw	0x0f
	addlw	0x01
	movwf	offset
	movlw	low hexdata 	;load low part of current veloc table
	addwf	offset,f		;add measured velocity
	movlw	high hexdata 	;load high part of current veloc table
	btfsc	STATUS,C		;if overflow then add 1
	addlw	0x01
	movwf	PCLATH			;load high address in lath
	movf	offset,w		;load computed offset
	call	hexdata			;call table and receive vaalue in w
	clrf	PCLATH			; back to Page 0
	call	LDisp_send_data
	;low nibble of error
	movf	errflags,w
	andlw	0x0f
	addlw	0x01
	movwf	offset
	movlw	low hexdata 	;load low part of current veloc table
	addwf	offset,f		;add measured velocity
	movlw	high hexdata 	;load high part of current veloc table
	btfsc	STATUS,C		;if overflow then add 1
	addlw	0x01
	movwf	PCLATH			;load high address in lath
	movf	offset,w		;load computed offset
	call	hexdata			;call table and receive vaalue in w
	clrf	PCLATH			; back to Page 0	
	call	LDisp_send_data
	goto	disp_msg_end
disp_msg_2
	Lstrout	msg_norun
	goto	disp_msg_end
disp_msg_end
	return

hexdata
	movwf	PCL
	dt	"0123456789ABCDEF"

;----
disp_stats					;displays stati on line 2..4
;2nd line
	movlw	b'11000000'		;cursor at pos 1 at 2nd line
	call	LDisp_send_conf
	movlw	'1'		;
	call	LDisp_send_data
	btfss	stat1,1
	goto	disp_s11
	Lstrout	msg_a
	goto	disp_s12
disp_s11
	Lstrout	msg_m
disp_s12
	btfss	stat1,0
	goto	disp_s13
	Lstrout	msg_on
	goto	disp_s14
disp_s13
	Lstrout	msg_off
disp_s14
	movlw	b'11001001'		;cursor at pos 10 at 2nd line	
	call	LDisp_send_conf
	movlw	'2'		;
	call	LDisp_send_data
	btfss	stat2,1
	goto	disp_s15
	Lstrout	msg_a
	goto	disp_s16
disp_s15
	Lstrout	msg_m
disp_s16
	btfss	stat2,0
	goto	disp_s17
	Lstrout	msg_on
	goto	disp_s18
disp_s17
	Lstrout	msg_off
disp_s18
;3rd line
	movlw	b'10010000'		;cursor at pos 1 at 3rd line
	call	LDisp_send_conf
	movlw	'3'		;
	call	LDisp_send_data
	btfss	stat3,1
	goto	disp_s21
	Lstrout	msg_a
	goto	disp_s22
disp_s21
	Lstrout	msg_m
disp_s22
	btfss	stat3,0
	goto	disp_s23
	Lstrout	msg_on
	goto	disp_s24
disp_s23
	Lstrout	msg_off
disp_s24
	movlw	b'10011001'		;cursor at pos 10 at 3rd line	
	call	LDisp_send_conf
	movlw	'4'		;
	call	LDisp_send_data
	btfss	stat4,1
	goto	disp_s25
	Lstrout	msg_a
	goto	disp_s26
disp_s25
	Lstrout	msg_m
disp_s26
	btfss	stat4,0
	goto	disp_s27
	Lstrout	msg_on
	goto	disp_s28
disp_s27
	Lstrout	msg_off
disp_s28
;4th line
	movlw	b'11010000'		;cursor at pos 1 at 4th line
	call	LDisp_send_conf
	movlw	'R'		;
	call	LDisp_send_data
	movlw	':'		;
	call	LDisp_send_data
	btfss	statn,0
	goto	disp_s33
	Lstrout	msg_on
	goto	disp_s34
disp_s33
	Lstrout	msg_off
disp_s34
	movlw	b'11011001'		;cursor at pos 10 at 4th line	
	call	LDisp_send_conf
	movlw	'P'		;
	call	LDisp_send_data
	btfss	statp,1
	goto	disp_s35
	Lstrout	msg_a
	goto	disp_s36
disp_s35
	Lstrout	msg_m
disp_s36
	btfss	statp,0
	goto	disp_s37
	Lstrout	msg_on
	goto	disp_s38
disp_s37
	Lstrout	msg_off
disp_s38
	return
;------------------------------------------------------------------------
;long call routines to routines on page 1
LDisp_send_conf
	movwf	tmp					;save W
	movlw	high Disp_send_conf
	movwf	PCLATH
	movf	tmp,w				;get w back
	call	Disp_send_conf		;call to Page 1
	clrf	PCLATH				;correct for Page 0
	return
LDisp_send_data
	movwf	tmp					;save W
	movlw	high Disp_send_data
	movwf	PCLATH
	movf	tmp,w				;get w back
	call	Disp_send_data		;call to Page 1
	clrf	PCLATH				;correct for Page 0
	return
LDelay100us
	movlw	high Delay100us
	movwf	PCLATH
	call	Delay100us
	clrf	PCLATH
	return
LDelay10ms
	movlw	high Delay10ms
	movwf	PCLATH
	call	Delay10ms
	clrf	PCLATH
	return
LDelay100ms
	movlw	high Delay100ms
	movwf	PCLATH
	call	Delay100ms
	clrf	PCLATH
	return
Lgetkey
	movlw	high getkey
	movwf	PCLATH
	call	getkey
	clrf	PCLATH
	return
Lkeycheck
	movlw	high keycheck
	movwf	PCLATH
	call	keycheck
	clrf	PCLATH
	return

Lwaitforrelease
	movlw	high waitforrelease
	movwf	PCLATH
	call	waitforrelease
	clrf	PCLATH
	return

Lbin2asc
	movwf	tmp					;save W
	movlw	high bin2asc
	movwf	PCLATH
	movf	tmp,w	
	call	bin2asc
	clrf	PCLATH
	return
;---------------------------------------------------------------
; humudity measurement
;---------------------------------------------------------------
measure_hum
	bcf		humerr			;no error at start
	bsf		statusexp,6		;power to humidity sensor
	movf	statusexp,w
	call	expander_wr
	call	LDelay100ms		;to stabilize power
	call	LDelay100ms
	call	handle_tmr1		;get current frequency
	bcf		statusexp,6		;power to humidity sensor off
	movf	statusexp,w
	call	expander_wr

measure_hum_1
	movf	cal2h,w			;calculate diff
	movwf	f1
	movf	cal2l,w
	movwf	f0
	movf	Hi,w
	movwf	f3
	movf	Lo,w
	movwf	f2
	call	sub16			;result in f1,f0
	clrf	f2				;we dont need f2
	movf	caldelta,w
	movwf	y
	call	FXD248U			;calculate, result in f2..f0
	movf	f0,w
	sublw	d'110'
	skpnc                  ;value<=110? (tolerance)
	goto	measure_hum_2    ;yes, then go on
	bsf		humerr			;else set error-flag
	clrf	hum             ;and set hum to 0% to be sure (obviously an error)
	return

measure_hum_2               ;otherwise check if <= 100%
	movf    f0,w
	sublw   d'100'
	skpnc
	goto    measure_hum_3   ;yes, go on below
	bsf     humerr          ;no, set indicator
	movlw   d'100'			;and clamp value to 100%
	movwf   f0

measure_hum_3
    movf    f0,w
    movwf   hum
	return
;---------------------------------------------------------------
;timer 1 handling routine - measures frequency on CLKIn
;frequency binary decoded 16 bit in Hi:Lo
;---------------------------------------------------------------
handle_tmr1
	
	bcf		T1CON,TMR1ON	;stop timer
	clrf	TMR1H			;clear tmr1 reg
	clrf	TMR1L
	btfss	seccounter,0	;wait for 1->0 transision of seccounter
	goto	handle_tmr1_a			;purpose: constant measuring time
	btfsc	seccounter,0
	goto	$-1
	goto	handle_tmr1_b
handle_tmr1_a
	btfss	seccounter,0	;wait for 0-> transition
	goto	$-1			
handle_tmr1_b
	clrf	keytimer		;start measuring 1 second
	bsf		T1CON,TMR1ON	;start timer
	btfss	keytimer,4		;wait until keytimer=16 (1sec over)
	goto	$-1
	bcf		T1CON,TMR1ON	;stop timer
	movf	TMR1H,w
	movwf	Hi
	movf	TMR1L,w
	movwf	Lo
		
	return
;---------------------------------------------------------------
;EEPROM handling routines
;---------------------------------------------------------------
;eprom read: Address in eprom_adr, return value in w
;---------------------------------------------------------------
eprom_read
	bcf		PIR2, EEIF		;clear interrupt flag (necessary?)
	movf	eprom_adr,w		;Set adress of read
	banksel	EEADR
	movwf	EEADR
	
	banksel	EECON1
	bcf		EECON1,EEPGD	;we want to read data mem
	bsf		EECON1,RD		;init read
	banksel	EEDATA
	movf	EEDATA,w
	bcf		STATUS,RP0		;back to bank 0 (our standard)
	bcf		STATUS,RP1
	return
;-------------------------------------------------------------------------
;eprom write Address in eprom_adr, data in eprom_dat returns when finished
;-------------------------------------------------------------------------
eprom_write
	bcf		PIR2, EEIF		;clear interrupt (to be sure)
	movf	eprom_adr,w		;load Adress
	banksel	EEADR
	movwf	EEADR
	banksel	0x00
	movf	eprom_dat,w	;load value
	banksel	EEDATA
	movwf	EEDATA
	banksel	EECON1
	bcf		EECON1,EEPGD	;we want to write data mem
	bsf		EECON1,WREN		;enable write
	
	bcf		INTCON,GIE		;we don't want interrupts now
	btfsc	INTCON,GIE		;make sure it is really cleared
	goto	$-2
	movlw	0x55			;see datasheet, required sequence
	movwf	EECON2
	movlw	0xaa
	movwf	EECON2
	bsf		EECON1,WR		;set write bit to begin write
	bsf		INTCON,GIE		;re-enable global interrupts
	
	banksel	0x00			;back to Bank0 (our default)
	btfss	PIR2,EEIF		;wait for flag to be set (polling)
	goto	$-1
	bcf		PIR2,EEIF		;clear it, write complete

	banksel	EECON1
	bcf		EECON1,WREN		;clear write enable
	banksel	0x00

	return
;--------------------------------------------------------------------
;Math routines
;--------------------------------------------------------------------
;--------------------------------------------------------------------
;sub 16
;performs [f1|f0] = [f1|f0] - [f3|f2] while keeping f3|f2 intact.
;(f0 and f2 being the low-bytes, f1 and f3 the hi-bytes)
;--------------------------------------------------------------------
sub16
	movf	f2,W
	subwf	f0,F		;f0-f2 (STATUS,C=!BORROW)
	movf	f3,W
	btfss	STATUS,C	;skip if borrow=0 (C=1)
	addlw	01
	subwf	f1,F		;f1-f3	STATUS,C=!BRW
	btfsc	STATUS,Z	;skip if MSbyte<>zero
	movf	f0,F		;else, check LSbyte for zero.
	return
;***********************************************************
;Unsigned 24 bit by 8 bit divide routine
;
; Inputs:
;   Dividend  - f0,f1,f2 (f2 - most significant!)
;   Divisor   - y
; Temporary:
;   Counter   - counter
; Output:
;   Quotient  - f0,f1,f2 (f2 - most significant!)
;   Remainder - f3
;
; This is basically Nikolai Golovchenko's 24 by 16 bit 
; divide routine, with some instructions removed to 
; optimise it for an 8 bit divide. 
; Thanks to Nikolai for the original post.
;
; James Hillman, 2 December 2005 
; Adapted to my vars /Stephan Asmus 4/2011
;***********************************************************

FXD248U:
    CLRF    f3          ;remainder
    MOVLW   d'24'
    MOVWF   counter

LOOPU248
    RLF     f0,W      ;shift dividend left to move next bit to remainder
    RLF     f1,F    ;
    RLF     f2,F    ;
    RLF     f3,F    ;shift carry (next dividend bit) into remainder
    RLF     f0,F      ;finish shifting the dividend and save carry in f0.0,
                     ;since remainder can be 9 bit long in some cases
                     ;This bit will also serve as the next result bit.
         
    MOVF    y,W      ;substract divisor from 8-bit remainder
    SUBWF   f3,F        

;here we also need to take into account the 9th bit of remainder, which
;is in f0.0. If we don't have a borrow after subtracting from 
;8 bits of remainder, then there is no borrow regardless of 9th bit 
;value. But, if we have the borrow, then that will depend on 9th bit 
;value. If it is 1, then no final borrow will occur. If it is 0, borrow
;will occur. These values match the borrow flag polarity.

    BTFSC   STATUS,0  ;if no borrow after 8 bit subtraction
    BSF     f0,0       ;then there is no borrow in result. Overwrite
                      ;f0.0 with 1 to indicate no borrow.
                      ;if borrow did occur, f0.0 already
                      ;holds the final borrow value (0-borrow, 
                      ;1-no borrow)

    BTFSS   f0,0       ;if no borrow after 9-bit subtraction
    ADDWF   f3,F     ;restore remainder. (w contains the value 
                      ;subtracted from it previously)
    DECFSZ  counter,F    
    GOTO    LOOPU248    
    RETURN
;--------------------------------------------------------------------
;Conversion routines
;--------------------------------------------------------------------
;--------------------------------------------------------------------
; BCD to Binary
; input:f0
; output:w
; From: Scott Dattalo
;--------------------------------------------------------------------
bcd2bin
        swapf   f0, W
        andlw   0x0F            ; W=tens
        movwf   tmp
        addwf   tmp, W         ; W=2*tens
        addwf   tmp, F         ; tmp=3*tens (note carry is cleared)
        rlf     tmp, W         ; W=6*tens
        subwf   f0, W          ; W = 16*tens+ones - 6*tens
	return
;--------------------------------------------------------------------
; bin to BCD conversion - limited to 0-99 decimal (0x00-0x63 hex)
; Call with: number to be converted in W
; Returns with: converted BCD number in W
; Registers: tmp, tmp1 used during conversion
;--------------------------------------------------------------------
bin2bcd
		movwf	tmp		;original hex, save to tmp
		clrw				
		btfsc	tmp, 4		;if 16 then bcd+16
		addlw	0x16
		btfsc	tmp, 5		;if 32 then bcd+32
		addlw	0x32
		addlw	0x06		;add 6 to correct bcd
		btfss	STATUS,DC	;overflow? then skip
		addlw	-0x06		;else subtract 6 again
		btfsc	tmp, 6		;if 64 then bcd+64
		addlw	0x64
		addlw	0x06		;bcd correction
		btfss	STATUS,DC
		addlw	-0x06
		movwf	tmp1		;save w to tmp1
		movf	tmp, 0		;load original hex to w
		andlw	0x0F		;mask ones
		addwf	tmp1, 0	;add ones to tmp1
		btfsc	STATUS,DC
		addlw	0x06		;bcd correction
		addlw	0x06		;bcd correction again
		btfss	STATUS,DC
		addlw	-0x06
		return
;--------------------------------------------------------------------
; bin to ascii conversion - 16bit
; Call with: number to be converted in Hi,Lo
; Returns with: ascii in asciiTK...asciiO
; Registers: temp
;--------------------------------------------------------------------
;Binary to ASCII, 16 bit to 5 digits (1 at a time) no temp register (!)
;by Nikolai Golovchenko
bin162ascii
    clrf	temp
sub30k
    movlw	d'3'
    addwf	temp, f
    movlw	low(d'30000')
    subwf	Lo, f

    movlw	high(d'30000')
    skpc				;C=1 bei w<=f
    movlw	high(d'30000') + 1
    subwf	Hi, f

    skpnc
    goto	sub30k

add10k
    decf	temp, f
    movlw	low(d'10000')
    addwf	Lo, f

    movlw	high(d'10000')
    skpnc
    movlw	high(d'10000') + 1
    addwf	Hi, f

    skpc
    goto	add10k
;   Output(temp)         ;output temp = TenThou
	movf	temp,w
	addlw	a'0'
	movwf	asciiTK

    clrf	temp
sub3k
    movlw	d'3'
    addwf	temp, f
    movlw	low(d'3000')
    subwf	Lo, f

    movlw	high(d'3000')
    skpc
    movlw	high(d'3000') + 1
    subwf	Hi, f

    skpnc
    goto	sub3k
add1k
    decf	temp, f
    movlw	low(d'1000')
    addwf	Lo, f

    movlw	high(d'1000')
    skpnc
    movlw	high(d'1000') + 1
    addwf	Hi, f

    skpc
    goto	add1k
;   Output(temp)         ;output temp = Thou
	movf	temp,w
	addlw	a'0'
	movwf	asciiK

    clrf	temp
sub300
    movlw	d'3'
    addwf	temp, f
    movlw	low(d'300')
    subwf	Lo, f

    movlw	high(d'300')
    skpc
    movlw	high(d'300') + 1
    subwf	Hi, f

    skpnc
    goto	sub300
    movlw	d'100'
add100
    decf	temp, f
    addwf	Lo, f
    skpc
    goto	add100
    incf	Hi, f
    btfsc	Hi, 7
    goto	add100
;   Output(temp)         ;output temp = Hund
	movf	temp,w
	addlw	a'0'
	movwf	asciiH

    clrf	temp
	movlw	d'30'
sub30
	incf	temp, f
    subwf 	Lo, f
    skpnc
   	goto	sub30
    movfw	temp
    rlf		temp, f
    addwf	temp, f
    movlw	d'10'
add10
    decf	temp, f
   	addwf	Lo, f
   	skpc
    goto	add10
;   Output(temp)         ;output temp = Tens
	movf	temp,w
	addlw	a'0'
	movwf	asciiT
;   Output(Lo)         ;output temp = Ones
	movf	Lo,w
	addlw	a'0'
	movwf	asciiO

	return
;--------------------------------------------------------------------
;PCF8574 Write
;Byte to output is in w
;--------------------------------------------------------------------
expander_wr
	bcf		tmpGIE,0
	btfss	INTCON,GIE
	goto	expander_wr_1
	bsf		tmpGIE,0		;remember State of GIE
	bcf		INTCON,GIE		;we don't want interrupts now
	btfsc	INTCON,GIE		;make sure it is really cleared
	goto	$-2
expander_wr_1
	movwf	tmp1			;save to tmp
	call	i2c_start
	movlw	0x40			; slave address + write
	call	i2c_write
	movf	tmp1,w
	call	i2c_write
	call	i2c_stop

	btfsc	tmpGIE,0			;restore GIE
	bsf		INTCON,GIE

	return
;--------------------------------------------------------------------
;RTC Subroutines
;DS1307
;--------------------------------------------------------------------
rtc_init
	bcf		tmpGIE,0
	btfss	INTCON,GIE
	goto	rtc_init_1
	bsf		tmpGIE,0		;remember State of GIE
	bcf		INTCON,GIE		;we don't want interrupts now
	btfsc	INTCON,GIE		;make sure it is really cleared
	goto	$-2
rtc_init_1
	call	i2c_start
	movlw	0xD0			; slave address + write
	call	i2c_write
	movlw	0x07			; set word address to control register
	call	i2c_write
	movlw	0x00				;write 0 to control reg
	call	i2c_write
	call	i2c_stop
	btfsc	tmpGIE,0			;restore GIE
	bsf		INTCON,GIE
	return
	
rtc_brst_rd
	bcf	tmpGIE,0
	btfss	INTCON,GIE
	goto	rtc_brst_rd_1
	bsf	tmpGIE,0		;remember State of GIE
	bcf		INTCON,GIE		;we don't want interrupts now
	btfsc	INTCON,GIE		;make sure it is really cleared
	goto	$-2
rtc_brst_rd_1

	call	i2c_start
	movlw	0xD0			; slave address + write
	call	i2c_write
	movlw	0			; set word address to seconds register
	call	i2c_write
	call	i2c_start
	movlw	0xD1			; slave address + read
	call	i2c_write
	call	i2c_read		; read the seconds data
	andlw	0x7f		;set bit 7 to 0
	movwf	sec			; save it
	call	i2c_ack
	call	i2c_read		; and so on
	movwf	min
	call	i2c_ack
	call	i2c_read
	andlw	0x3f		;set bit 7 and 6 to 0 
	movwf	hr
	call	i2c_nack
	call	i2c_stop
	btfsc	tmpGIE,0			;restore GIE
	bsf	INTCON,GIE
	return

rtc_brst_wr
	bcf		tmpGIE,0
	btfss	INTCON,GIE
	goto	rtc_brst_wr_1
	bsf		tmpGIE,0		;remember State of GIE
	bcf		INTCON,GIE		;we don't want interrupts now
	btfsc	INTCON,GIE		;make sure it is really cleared
	goto	$-2
rtc_brst_wr_1

	call	i2c_start
	movlw	0xD0			; slave address + write
	call	i2c_write
	movlw	0			; set word address to seconds register
	call	i2c_write
	movf	sec, W
	call	i2c_write
	movf	min, W
	call	i2c_write
	movf	hr, W
	call	i2c_write
	call	i2c_stop
	btfsc	tmpGIE,0			;restore GIE
	bsf	INTCON,GIE
	return
;--------------------------------------------------------------------
;I2C Subroutines
;--------------------------------------------------------------------
i2c_start
	bsf		SDA			; SDA high
	;SET_BANK1
	;bcf		SDA_T		; set SDA for output
	nop
	SET_BANK1
	bsf		SCL_T		; SCL high (input)
	SET_BANK0
	bcf		SDA			; SDA low (start)
	return
;-------------	
i2c_stop; assumes SCL is high on entry
	bcf		SDA			; SDA low
	nop
	nop
	SET_BANK1
	bsf		SCL_T		; SCL high (input)
	SET_BANK0
	bsf		SDA			; SDA high
	return
;-------------
;i2c_read
;returns read byte in w
;-------------
i2c_read
	SET_BANK1
	bsf		SDA_T		; set SDA for input
	SET_BANK0

	movlw	08h			; send 8 bits
	movwf	count

	bcf		SCL			; clock data out
	SET_BANK1
	bcf		SCL_T		; SCL low (output)
	SET_BANK0

	clrf	tmp			; clear var
	rlf		tmp,1			; rotate carry in
	clrf	tmp			; clear var again

i2c_read_loop
	rlf		tmp, 1

	SET_BANK1
	bsf		SCL_T		; SCL high (input)
	SET_BANK0

	btfsc	SDA
	bsf		tmp, 0			; if data out = 1, set bit

	bcf		SCL
	SET_BANK1
	bcf		SCL_T		; SCL low (output)
	SET_BANK0
	decfsz	count, 1
	goto	i2c_read_loop

	movf	tmp, W

	return

;---- ACK read (assumes SCL=0 on entry) ----
i2c_ack
	bcf		SDA

	SET_BANK1
	bcf		SDA_T		; set SDA for output
	SET_BANK0

	SET_BANK1
	bsf		SCL_T		; SCL high (input)
	SET_BANK0
	nop
	bcf		SCL
	SET_BANK1
	bcf		SCL_T		; SCL low (output)
	SET_BANK0

	return

;---- NACK read (assumes SCL = 0 on entry) ----
i2c_nack
	bsf		SDA

	SET_BANK1
	bcf		SDA_T		; set SDA for output
	SET_BANK0

	SET_BANK1
	bsf		SCL_T		; SCL high (input)
	SET_BANK0

	bcf	SCL
	SET_BANK1
	bcf		SCL_T		; SCL low (output)
	SET_BANK0

	return

;--- Do a I2C bus write of byte in 'tmp' ---
;
i2c_write
	movwf	tmp		;save tmp

	SET_BANK1
	bcf		SDA_T		; set SDA for output
	SET_BANK0

	movlw	08h			; send 8 bits
	movwf	count

	bcf		SCL
	SET_BANK1
	bcf		SCL_T		; SCL low (output)
	SET_BANK0

i2c_w_loop
	bcf		SDA			; assume data out is low
	btfsc	tmp, 7
	bsf		SDA			; if data out = 1, set bit
	; nop

	SET_BANK1
	bsf		SCL_T		; SCL high (input)
	SET_BANK0
	rlf		tmp, 1
	bcf		SCL			; clock it in
	SET_BANK1
	bcf		SCL_T		; SCL low (output)
	SET_BANK0
	decfsz	count, 1
	goto	i2c_w_loop

	SET_BANK1
	bsf		SDA_T	; set SDA for input
	SET_BANK0

	bcf		SCL
	SET_BANK1
	bcf		SCL_T		; SCL low (output)
	SET_BANK0
	; nop
	SET_BANK1
	bsf		SCL_T		; SCL high (input)
	SET_BANK0
	; if(sda) printf("Ack bit missing  %02X\n",(unsigned int)d);
	bcf		SCL
	SET_BANK1
	bcf		SCL_T		; SCL low (output)
	SET_BANK0

	return

;---------------------------------------------------------------------------
; Message definitions for display (1)
; more Definitions see end of Code Page 2
;---------------------------------------------------------------------------
msg_welcome		dt	"Bew",0xE1,"sserung V0.3",0x00
msg_welcome1	dt	"(C) SASMUS 2014 ",0x00

msg_a			dt	":a-",0x00
msg_m			dt	":m-",0x00
msg_on			dt	"on ",0x00
msg_off			dt	"off",0x00

msg_mainmenu	dt	"----Mainmenu----",0x00
msg_start		dt	"|Start Program >",0x00
msg_stop		dt	"<Stop Program  >",0x00
msg_pgm			dt	"<Program edit  >",0x00
msg_hum			dt	"<Hum sensor    >",0x00
msg_refill		dt	"<Refill system >",0x00
msg_clock		dt	"<Set Clock     |",0x00

msg_pgm_menu	dt	"-Program config-",0x00
msg_pgm_onoff	dt  "|Program on/off>",0x00
msg_pgm_on		dt	"Program On  ",0x00
msg_pgm_off		dt	"Program Off ",0x00
msg_pgm_start	dt	"<Pgm. Starttime>",0x00
msg_pgm_dur		dt	"<Pgm. Duration |",0x00
msg_pgm_pgmdur	dt	"Program X-XXmin ",0x00

;########################################################################
	org	0x800	;define next in memory Page 1 (we need space in Page 0)
;########################################################################	
;---------------------------------------------------------------	
;Menu handling
;---------------------------------------------------------------
handle_mainmenu
	call	Disp_clear		;clear display
	call	Delay10ms		;needs time to finish...
	movlw	high disp_time
	movwf	PCLATH
	call	disp_time		;update Display 1st row
	call	disp_hum
	call	disp_msg
	movlw	high handle_mainmenu
	movwf	PCLATH
	movlw	b'11000000'		;cursor at pos 1 at 2nd line
	call	Disp_send_conf
	strout	msg_mainmenu	;now menu message on display
hm_start
	bsf		Backlight		;key pressed, Backlight on
	clrf	bltimer
	movlw	b'10010000'		;cursor at pos 1 at 3rd line
	call	Disp_send_conf
	strout 	msg_start		;first one in 3rd row
hm_start_1
	call	waitforrelease	;wait until no key pressed
	call	getkey			;get new status
	XORLW	0x04			;X? (exit)
	skpnz
	return					;then leave mainmenu
	;movf	keydata,w		
	xorlw	0x05^0x04		;ok pressed?
	skpz					
	goto	hm_start_2
	bsf		pgmrun			;then set flag run program
	bsf		manstartstop
	return					;and leave Mainmenu
hm_start_2
	;movf	keydata,w		
	xorlw	0x03^0x05			;up pressed?
	skpz
	goto	hm_start_1		;else do it again
	;we get here if up was pressed...
hm_stop
	bsf		Backlight		;key pressed, Backlight on
	clrf	bltimer
	movlw	b'10010000'		;cursor at pos 1 at 3rd line
	call	Disp_send_conf
	strout 	msg_stop		;first one in 3rd row
hm_stop_1
	call	waitforrelease	;wait until no key pressed
	call	getkey			;get new status
	XORLW	0x04			;X? (exit)
	skpnz
	return					;then leave mainmenu
	;movf	keydata,w		
	xorlw	0x05^0x04			;ok pressed?
	skpz					
	goto	hm_stop_2
	bcf		pgmrun			;then clear flag run program
	bsf		manstartstop	;manual pressed stop
	movlw	0x05			;set phase to 5 (=end)
	movwf	pgmphase
	return					;and leave Mainmenu
hm_stop_2
	;movf	keydata,w		
	xorlw	0x06^0x05		;down pressed?
	skpnz
	goto	hm_start		;previous menu entry
	;movf	keydata,w		
	xorlw	0x03^0x06		;up pressed?
	skpz
	goto	hm_stop_1		;previous menu entry
	;we get here if up was pressed...
hm_pgm
	bsf		Backlight		;key pressed, Backlight on
	clrf	bltimer
	movlw	b'10010000'		;cursor at pos 1 at 3rd line
	call	Disp_send_conf
	strout 	msg_pgm			;first one in 3rd row
hm_pgm_1
	call	waitforrelease	;wait until no key pressed
	call	getkey			;get new status
	XORLW	0x04			;X? (exit)
	skpnz
	return					;then leave mainmenu
	;movf	keydata,w		
	xorlw	0x05^0x04		;ok pressed?
	skpnz					
	goto	hsm_pgm
	;movf	keydata,w		
	xorlw	0x06^0x05		;down pressed?
	skpnz
	goto	hm_stop			;previous menu entry
	;movf	keydata,w		
	xorlw	0x03^0x06		;up pressed?
	skpz
	goto	hm_pgm_1		;previous menu entry
	;we get here if up was pressed...
hm_hum
	bsf		Backlight		;key pressed, Backlight on
	clrf	bltimer
	movlw	b'10010000'		;cursor at pos 1 at 3rd line
	call	Disp_send_conf
	strout 	msg_hum			;first one in 3rd row
hm_hum_1
	call	waitforrelease	;wait until no key pressed
	call	getkey			;get new status
	XORLW	0x04			;X? (exit)
	skpnz
	return					;then leave mainmenu
	;movf	keydata,w		
	xorlw	0x05^0x04		;ok pressed?
	skpnz					
	goto	hsm_hum
	;movf	keydata,w		
	xorlw	0x06^0x05			;down pressed?
	skpnz
	goto	hm_pgm			;previous menu entry
	;movf	keydata,w		
	xorlw	0x03^0x06		;up pressed?
	skpz
	goto	hm_hum_1		;previous menu entry
	;we get here if up was pressed...
hm_ref
	bsf		Backlight		;key pressed, Backlight on
	clrf	bltimer
	movlw	b'10010000'		;cursor at pos 1 at 3rd line
	call	Disp_send_conf
	strout 	msg_refill		;first one in 3rd row
hm_ref_1
	call	waitforrelease	;wait until no key pressed
	call	getkey			;get new status
	XORLW	0x04			;X? (exit)
	skpnz
	return					;then leave mainmenu
	;movf	keydata,w		
	xorlw	0x05^0x04			;ok pressed?
	skpnz					
	goto	hsm_ref			;then handle submenu
	;movf	keydata,w		
	xorlw	0x06^0x05		;down pressed?
	skpnz
	goto	hm_hum			;previous menu entry
	;movf	keydata,w		
	xorlw	0x03^0x06		;up pressed?
	skpz
	goto	hm_ref_1		;previous menu entry
	;we get here if up was pressed...
hm_clock
	bsf		Backlight		;key pressed, Backlight on
	clrf	bltimer
	movlw	b'10010000'		;cursor at pos 1 at 3rd line
	call	Disp_send_conf
	strout 	msg_clock		;first one in 3rd row
hm_clock_1
	call	waitforrelease	;wait until no key pressed
	call	getkey			;get new status
	XORLW	0x04			;X? (exit)
	skpnz
	return					;then leave mainmenu
	;movf	keydata,w		
	xorlw	0x05^0x04		;ok pressed?
	skpnz					
	goto	hsm_clock
	;movf	keydata,w		
	xorlw	0x06^0x05		;down pressed?
	skpnz
	goto	hm_ref			;previous menu entry
	goto	hm_clock_1		;last menu entry

	;return					;not necessary because return is handled in sub
;-----	
;-----------Submenu Program----------------------------
hsm_pgm

	movlw	b'11000000'		;cursor at pos 1 at 2nd line
	call	Disp_send_conf
	strout	msg_pgm_menu
	movlw	b'10010000'		;cursor at pos 1 at 3rd line
	call	Disp_send_conf
	strout	msg_pgm_onoff
	movlw	b'11010000'		;cursor at pos 1 at 4th line
	call	Disp_send_conf
	strout	msg_empty
hsm_pgm_1
	call	waitforrelease	;wait until no key pressed
	call	getkey			;get new status
	XORLW	0x04			;X? (exit)
	skpnz
	goto	hsm_pgm_end
	;movf	keydata,w
	xorlw	0x03^0x04		;up?
	skpnz
	goto	hsm_pgmA		;yes, next menu entry
	;movf	keydata,w
	xorlw	0x05^0x03		;ok?
	skpz
	goto	hsm_pgm_1
hsm_pgm_11					;ok pressed
	bsf		confchange
	movlw	b'11010000'		;cursor at pos 1 at 4th line
	call	Disp_send_conf
	btfsc	pgmuse
	goto	hsm_pgm_12
	strout	msg_pgm_off
hsm_pgm_12
	btfss	pgmuse
	goto	hsm_pgm_13
	strout	msg_pgm_on
hsm_pgm_13
	call	waitforrelease
	call 	getkey
	xorlw	0x05			;ok?
	skpnz
	goto	hsm_pgm			;yes, back to menu
	;movf	keydata,w
	xorlw	0x03^0x05		;up?
	skpnz	
	bsf		pgmuse			;yes, set on
	;movf	keydata,w
	xorlw	0x06^0x03		;down
	skpnz
	bcf		pgmuse			;yes, then set off
	goto	hsm_pgm_11
;---------pgm starttime submenu------------
hsm_pgmA					
	movlw	b'00001100'		;set cursor blinking off
	call	Disp_send_conf
	movlw	b'10010000'		;cursor at pos 1 at 3rd line
	call	Disp_send_conf
	strout	msg_pgm_start
	movlw	b'11010000'		;cursor at pos 1 at 4th line
	call	Disp_send_conf
	strout	msg_empty
hsm_pgmA_1
	call	waitforrelease	;wait until no key pressed
	call	getkey			;get new status
	XORLW	0x04			;X? (exit)
	skpnz
	goto	hsm_pgm_end
	;movf	keydata,w
	xorlw	0x03^0x04		;up?
	skpnz
	goto	hsm_pgmB		;yes, next menu entry
	;movf	keydata,w
	xorlw	0x06^0x03		;down?
	skpnz
	goto	hsm_pgm
	;movf	keydata,w
	xorlw	0x05^0x06		;ok?
	skpz
	goto	hsm_pgmA_1
	movlw	b'00001101'		;set cursor blinking
	call	Disp_send_conf
hsm_pgmA_11					;ok pressed
	bsf		confchange
	movlw	b'11010111'		;cursor at pos 8 at 4th line
	call	Disp_send_conf
	movlw	':'			
	call	Disp_send_data
	movf	startmin,w		;Minutes to display
	call	bin2asc
	movf	asciiT,w
	call	Disp_send_data
	movf	asciiO,w
	call	Disp_send_data
hsm_pgmA_12
	movlw	b'11010101'		;cursor at pos 6 at 4th line
	call	Disp_send_conf
	movf	starthr,w		;Hours to display
	call	bin2asc
	movf	asciiT,w
	call	Disp_send_data
	movf	asciiO,w
	call	Disp_send_data
	movlw	b'11010110'		;cursor at pos 7 at 4th line (for blink)
	call	Disp_send_conf
	call	waitforrelease
	call 	getkey
	xorlw	0x05			;ok?
	skpnz
	goto	hsm_pgmA_14		;yes, edit minutes
	;movf	keydata,w
	xorlw	0x03^0x05		;up?
	skpz	
	goto	hsm_pgmA_13		;no
	incf	starthr,f		;increment hrs
	movf	starthr,w
	xorlw	d'24'			;>23?
	skpz
	goto	hsm_pgmA_13
	movlw	d'23'			;yes, then set to 23
	movwf	starthr
hsm_pgmA_13
	movf	keydata,w
	xorlw	0x06			;down?
	skpz	
	goto	hsm_pgmA_12		;no
	decf	starthr,f		;yes, decrement hrs
	movf	starthr,w
	xorlw	d'255'			;<0?
	skpz
	goto	hsm_pgmA_12
	movlw	d'0'			;yes, then set to 0
	movwf	starthr
	goto	hsm_pgmA_12
hsm_pgmA_14					;****Edit Minutes
	movlw	b'11011000'		;cursor at pos 8 at 4th line
	call	Disp_send_conf
	movf	startmin,w		;Hours to display
	call	bin2asc
	movf	asciiT,w
	call	Disp_send_data
	movf	asciiO,w
	call	Disp_send_data
	movlw	b'11011001'		;cursor at pos 9 at 4th line (for blink)
	call	Disp_send_conf
	call	waitforrelease
	call 	getkey
	xorlw	0x05			;ok?
	skpnz
	goto	hsm_pgmA		;yes, back to submenu
	;movf	keydata,w
	xorlw	0x03^0x05		;up?
	skpz	
	goto	hsm_pgmA_15		;no
	incf	startmin,f		;increment mins
	movf	startmin,w
	xorlw	d'60'			;>59?
	skpz
	goto	hsm_pgmA_15
	movlw	d'59'			;yes, then set to 59
	movwf	startmin
hsm_pgmA_15
	movf	keydata,w
	xorlw	0x06			;down?
	skpz	
	goto	hsm_pgmA_14		;no
	decf	startmin,f		;yes, decrement mins
	movf	startmin,w
	xorlw	d'255'			;<0?
	skpz
	goto	hsm_pgmA_14
	movlw	d'0'			;yes, then set to 0
	movwf	startmin
	goto	hsm_pgmA_14

;---------pgm duration submenu------------
hsm_pgmB		
	movlw	b'00001100'		;set cursor blinking off
	call	Disp_send_conf
	movlw	b'10010000'		;cursor at pos 1 at 3rd line
	call	Disp_send_conf
	strout	msg_pgm_dur
	movlw	b'11010000'		;cursor at pos 1 at 4th line
	call	Disp_send_conf
	strout	msg_empty
hsm_pgmB_1
	call	waitforrelease	;wait until no key pressed
	call	getkey			;get new status
	XORLW	0x04			;X? (exit)
	skpnz
	goto	hsm_pgm_end
	;movf	keydata,w
	xorlw	0x06^0x04		;down?
	skpnz
	goto	hsm_pgmA
	;movf	keydata,w
	xorlw	0x05^0x06		;ok?
	skpz
	goto	hsm_pgmB_1
	movlw	b'00001101'		;set cursor blinking
	call	Disp_send_conf
	movlw	0x01
	movwf	currdur			;set current duration =1
hsm_pgmB_11					;ok pressed
	bsf		confchange
	movlw	b'11010000'		;cursor at pos 0 at 4th line
	call	Disp_send_conf
	strout	msg_pgm_pgmdur
	movlw	b'11011000'		;cursor at pos 8 at 4th line
	call	Disp_send_conf
	movf	currdur,w		;currrent channel on Display
	call	bin2asc
	movf	asciiO,w
	call	Disp_send_data
	movlw	'-'
	call	Disp_send_data

	movf	currdur,w
	xorlw	0x01			;is it 1?
	skpz
	goto	hsm_pgmB_12		;no, then next check
	movf	dur1,w			;then duration 1 to w
	movwf	tmpdur
	goto	hsm_pgmB_15
hsm_pgmB_12
	;movf	currdur,w
	xorlw	0x02^0x01		;is it 2?
	skpz
	goto	hsm_pgmB_13		;no, then next check
	movf	dur2,w			;then duration 2 to w
	movwf	tmpdur
	goto	hsm_pgmB_15
hsm_pgmB_13
	;movf	currdur,w
	xorlw	0x03^0x02		;is it 3?
	skpz
	goto	hsm_pgmB_14		;no, then next check
	movf	dur3,w			;then duration 3 to w
	movwf	tmpdur
	goto	hsm_pgmB_15
hsm_pgmB_14					;then it has to be 4
	movf	dur4,w			;then duration 4 to w
	movwf	tmpdur
hsm_pgmB_15
	call	bin2asc
	movf	asciiT,w
	call	Disp_send_data
	movf	asciiO,w
	call	Disp_send_data

	movlw	b'11011000'		;cursor at pos 8 at 4th line
	call	Disp_send_conf
	call	waitforrelease
	call 	getkey
	xorlw	0x05			;ok?
	skpnz
	goto	hsm_pgmB_17		;yes, edit duration
	;movf	keydata,w
	xorlw	0x03^0x05		;up?
	skpz	
	goto	hsm_pgmB_16		;no
	incf	currdur,f		;increment current channel
	movf	currdur,w
	xorlw	d'5'			;>4?
	skpz
	goto	hsm_pgmB_16
	movlw	d'4'			;yes, then set to 4
	movwf	currdur
hsm_pgmB_16
	movf	keydata,w
	xorlw	0x06			;down?
	skpz	
	goto	hsm_pgmB_11		;no
	decf	currdur,f		;yes, decrement channel
	movf	currdur,w
	xorlw	d'0'			;<1?
	skpz
	goto	hsm_pgmB_11
	movlw	d'1'			;yes, then set to 1
	movwf	currdur
	goto	hsm_pgmB_11
	
hsm_pgmB_17					;****Edit Minutes
	movlw	b'11011010'		;cursor at pos 10 at 4th line
	call	Disp_send_conf
	movf	tmpdur,w		;Duration to display
	call	bin2asc
	movf	asciiT,w
	call	Disp_send_data
	movf	asciiO,w
	call	Disp_send_data
	movlw	b'11011011'		;cursor at pos 11 at 4th line (for blink)
	call	Disp_send_conf
	call	waitforrelease
	call 	getkey
	xorlw	0x05			;ok?
	skpnz
	goto	hsm_pgmB_19		;yes, save it 
	;movf	keydata,w
	xorlw	0x03^0x05		;up?
	skpz	
	goto	hsm_pgmB_18		;no
	incf	tmpdur,f		;increment duration
	movf	tmpdur,w
	xorlw	d'61'			;>60?
	skpz
	goto	hsm_pgmB_18
	movlw	d'60'			;yes, then set to 60
	movwf	tmpdur
hsm_pgmB_18
	movf	keydata,w
	xorlw	0x06			;down?
	skpz	
	goto	hsm_pgmB_17		;no
	decf	tmpdur,f		;yes, decrement duration
	movf	tmpdur,w
	xorlw	d'255'			;<0?
	skpz
	goto	hsm_pgmB_17
	movlw	d'0'			;yes, then set to 0
	movwf	tmpdur
	goto	hsm_pgmB_17
hsm_pgmB_19					;now save it and back to menu
	movf	currdur,w
	xorlw	0x01			;is it 1?
	skpz
	goto	hsm_pgmB_20		;no, then next check
	movf	tmpdur,w
	movwf	dur1
	goto	hsm_pgmB
hsm_pgmB_20
	;movf	currdur,w
	xorlw	0x02^0x01		;is it 2?
	skpz
	goto	hsm_pgmB_21		;no, then next check
	movf	tmpdur,w
	movwf	dur2
	goto	hsm_pgmB
hsm_pgmB_21
	;movf	currdur,w
	xorlw	0x03^0x02		;is it 3?
	skpz
	goto	hsm_pgmB_22		;no, then next check
	movf	tmpdur,w
	movwf	dur3
	goto	hsm_pgmB
hsm_pgmB_22					;then it has to be 4
	movf	tmpdur,w
	movwf	dur4
	goto	hsm_pgmB
;------Submenu Pgm end
hsm_pgm_end
	movlw	b'11000000'		;cursor at pos 1 at 2nd line
	call	Disp_send_conf
	strout	msg_mainmenu
	goto	hm_pgm
;-----------Submenu Humidity----------------------------
hsm_hum
	movlw	b'11000000'		;cursor at pos 1 at 2nd line
	call	Disp_send_conf
	strout	msg_hum_menu
	movlw	b'10010000'		;cursor at pos 1 at 3rd line
	call	Disp_send_conf
	strout	msg_hum_usage
	movlw	b'11010000'		;cursor at pos 1 at 4th line
	call	Disp_send_conf
	strout	msg_empty
hsm_hum_1
	call	waitforrelease	;wait until no key pressed
	call	getkey			;get new status
	XORLW	0x04			;X? (exit)
	skpnz
	goto	hsm_hum_end
	;movf	keydata,w
	xorlw	0x03^0x04			;up?
	skpnz
	goto	hsm_humA		;yes, next menu entry
	;movf	keydata,w
	xorlw	0x05^0x03		;ok?
	skpz
	goto	hsm_hum_1
hsm_hum_11					;ok pressed
	bsf		confchange
	movlw	b'11010000'		;cursor at pos 1 at 4th line
	call	Disp_send_conf
	btfsc	humuse
	goto	hsm_hum_12
	strout	msg_hum_off
hsm_hum_12
	btfss	humuse
	goto	hsm_hum_13
	strout	msg_hum_on
hsm_hum_13
	call	waitforrelease
	call 	getkey
	xorlw	0x05			;ok?
	skpnz
	goto	hsm_hum			;yes, back to menu
	;movf	keydata,w
	xorlw	0x03^0x05		;up?
	skpnz	
	bsf		humuse			;yes, set on
	;movf	keydata,w
	xorlw	0x06^0x03		;down?
	skpnz
	bcf		humuse			;yes, then set off
	goto	hsm_hum_11
;--------Humidity Treshold
hsm_humA
	movlw	b'00001100'		;set cursor blinking off
	call	Disp_send_conf
	movlw	b'10010000'		;cursor at pos 1 at 3rd line
	call	Disp_send_conf
	strout	msg_hum_tres
	movlw	b'11010000'		;cursor at pos 1 at 4th line
	call	Disp_send_conf
	strout	msg_empty
hsm_humA_1
	call	waitforrelease	;wait until no key pressed
	call	getkey			;get new status
	XORLW	0x04			;X? (exit)
	skpnz
	goto	hsm_hum_end
	;movf	keydata,w
	xorlw	0x03^0x04		;up?
	skpnz
	goto	hsm_humB		;yes, next menu entry
	;movf	keydata,w
	xorlw	0x06^0x03		;down?
	skpnz
	goto	hsm_hum
	;movf	keydata,w
	xorlw	0x05^0x06		;ok?
	skpz
	goto	hsm_humA_1
	movlw	b'00001101'		;set cursor blinking
	call	Disp_send_conf
hsm_humA_11					;ok pressed
	bsf		confchange
	movlw	b'11010000'		;cursor at pos 0 at 4th line
	call	Disp_send_conf
	strout	msg_hum_tresval
hsm_humA_12
	movlw	b'11011001'		;cursor at pos 9 at 4th line
	call	Disp_send_conf
	movf	humtres,w		;Treshold to display
	call	bin2asc
	movf	asciiT,w
	call	Disp_send_data
	movf	asciiO,w
	call	Disp_send_data
	movlw	b'11011010'		;cursor at pos 10 at 4th line (for blink)
	call	Disp_send_conf
	call	waitforrelease
	call 	getkey
	xorlw	0x05			;ok?
	skpnz
	goto	hsm_humA		;yes, back to menu
	;movf	keydata,w
	xorlw	0x03^0x05		;up?
	skpz	
	goto	hsm_humA_13		;no
	incf	humtres,f		;increment Treshold
	movf	humtres,w
	xorlw	d'100'			;>99?
	skpz
	goto	hsm_humA_13
	movlw	d'99'			;yes, then set to 99
	movwf	humtres
hsm_humA_13
	movf	keydata,w
	xorlw	0x06			;down?
	skpz	
	goto	hsm_humA_12		;no
	decf	humtres,f		;decrement treshold
	movf	humtres,w
	xorlw	d'255'			;<0?
	skpz
	goto	hsm_humA_12
	movlw	d'0'			;yes, then set to 0
	movwf	humtres
	goto	hsm_humA_12
;------------ Humidity probe interval
hsm_humB
	movlw	b'00001100'		;set cursor blinking off
	call	Disp_send_conf
	movlw	b'10010000'		;cursor at pos 1 at 3rd line
	call	Disp_send_conf
	strout	msg_hum_int
	movlw	b'11010000'		;cursor at pos 1 at 4th line
	call	Disp_send_conf
	strout	msg_empty
hsm_humB_1
	call	waitforrelease	;wait until no key pressed
	call	getkey			;get new status
	XORLW	0x04			;X? (exit)
	skpnz
	goto	hsm_hum_end
	;movf	keydata,w
	xorlw	0x03^0x04		;up?
	skpnz
	goto	hsm_humC		;yes, next menu entry
	;movf	keydata,w
	xorlw	0x06^0x03		;down?
	skpnz
	goto	hsm_humA
	;movf	keydata,w
	xorlw	0x05^0x06		;ok?
	skpz
	goto	hsm_humB_1
	movlw	b'00001101'		;set cursor blinking
	call	Disp_send_conf
hsm_humB_11					;ok pressed
	bsf		confchange
	movlw	b'11010000'		;cursor at pos 0 at 4th line
	call	Disp_send_conf
	strout	msg_hum_intval
hsm_humB_12
	clrf	senstimer		;we change interval, so start counting at 0
	movlw	b'11011001'		;cursor at pos 9 at 4th line
	call	Disp_send_conf
	movf	humint,w		;Interval to display
	call	bin2asc
	movf	asciiT,w
	call	Disp_send_data
	movf	asciiO,w
	call	Disp_send_data

	movlw	b'11011010'		;cursor at pos 10 at 4th line (for blink)
	call	Disp_send_conf
	call	waitforrelease
	call 	getkey
	xorlw	0x05			;ok?
	skpnz
	goto	hsm_humB		;yes, back to menu
	;movf	keydata,w
	xorlw	0x03^0x05		;up?
	skpz	
	goto	hsm_humB_13		;no
	incf	humint,f		;increment Duration
	movf	humint,w
	xorlw	d'31'			;>30?
	skpz
	goto	hsm_humB_13
	movlw	d'30'			;yes, then set to 30
	movwf	humint
hsm_humB_13
	movf	keydata,w
	xorlw	0x06			;down?
	skpz	
	goto	hsm_humB_12		;no
	decf	humint,f		;yes, decrement interval
	movf	humint,w
	xorlw	d'0'			;<1?
	skpz
	goto	hsm_humB_12
	movlw	d'1'			;yes, then set to 1
	movwf	humint
	goto	hsm_humB_12
;------------ Humidity sensor calibration
hsm_humC
	movlw	high disp_msg
	movwf	PCLATH
	call	disp_msg		;update Error-Display
	movlw	high $
	movwf	PCLATH
	movlw	b'00001100'		;set cursor blinking off
	call	Disp_send_conf
	movlw	b'10010000'		;cursor at pos 1 at 3rd line
	call	Disp_send_conf
	strout	msg_hum_calib
	movlw	b'11010000'		;cursor at pos 1 at 4th line
	call	Disp_send_conf
	strout	msg_empty
hsm_humC_1
	call	waitforrelease	;wait until no key pressed
	call	getkey			;get new status
	XORLW	0x04			;X? (exit)
	skpnz
	goto	hsm_hum_end
	xorlw	0x06^0x04		;down?
	skpnz
	goto	hsm_humB
	xorlw	0x05^0x06		;ok?
	skpz
	goto	hsm_humC_1

hsm_humC_12
	movlw	high expander_wr
	movwf	PCLATH
    bsf		statusexp,6     ;power to sensor
	movf	statusexp,w
	call	expander_wr
	movlw	high $
	movwf	PCLATH
	;call	Delay100ms		;Wait to stabilize Power... (unnecessary now..
	;call	Delay100ms      ;because of key request below...)

	bsf		confchange
	movlw	b'11010000'		;cursor at pos 0 at 4th line
	call	Disp_send_conf
	strout	msg_hum_calwet
	call	waitforrelease
	call 	getkey
	XORLW	0x04			;X? (exit)
	skpnz
	goto	hsm_humC_13     ;goto Cal dry (override wet calibration)
	xorlw	0x05^0x04		;ok?
	skpz
	goto	hsm_humC_12

	;Code to handle calibration wet-value
	movlw	high handle_tmr1
	movwf	PCLATH
	call	handle_tmr1		;get frequency
	movf	Hi,w			;save to prefs
	movwf	cal1h
	movf	Lo,w
	movwf	cal1l
	call	bin162ascii		;transform to ascii for display
	movlw	high $
	movwf	PCLATH
	
	movlw	b'11010000'		;cursor at pos 0 at 4th line
	call	Disp_send_conf
	movf	asciiTK,w
	call	Disp_send_data
	movf	asciiK,w
	call	Disp_send_data
	movf	asciiH,w
	call	Disp_send_data
	movf	asciiT,w
	call	Disp_send_data
	movf	asciiO,w
	call	Disp_send_data
	strout	msg_hum_hz
	call 	getkey			;wait for ok

hsm_humC_13
	movlw	b'11010000'		;cursor at pos 0 at 4th line
	call	Disp_send_conf
	strout	msg_hum_caldry
	call	waitforrelease
	call 	getkey
	XORLW	0x04			;X? (exit)
	skpnz
	goto	hsm_humC_14     ;goto Cal computing (override dry calibration)
	xorlw	0x05^0x04		;ok?
	skpz
	goto	hsm_humC_13
	;code to handle calibration dry-value
	movlw	high handle_tmr1
	movwf	PCLATH
	call	handle_tmr1		;get frequency
	movf	Hi,w			;save to prefs
	movwf	cal2h
	movf	Lo,w
	movwf	cal2l
	call	bin162ascii		;transform to ascii for display
	movlw	high $
	movwf	PCLATH

	movlw	b'11010000'		;cursor at pos 0 at 4th line
	call	Disp_send_conf
	movf	asciiTK,w
	call	Disp_send_data
	movf	asciiK,w
	call	Disp_send_data
	movf	asciiH,w
	call	Disp_send_data
	movf	asciiT,w
	call	Disp_send_data
	movf	asciiO,w
	call	Disp_send_data
	strout	msg_hum_hz
	call 	getkey			;wait for ok

hsm_humC_14
	movlw	high expander_wr
	movwf	PCLATH
	bcf		statusexp,6		;power to sensor off
	movf	statusexp,w
	call	expander_wr

	movf	cal2h,w			;save to f1/f0 for sub16
	movwf	f1
	movf	cal2l,w
	movwf	f0
	movf	cal1h,w
	movwf	f3
	movf	cal1l,w
	movwf	f2
	bcf		caliberr		;clear error flag
	call	sub16			;compute diff, result in f1,f0
	skpc                    ;if not carry, then result is negative
	bsf		caliberr		;so indicate error
	bcf		LED
	movf	errflags,w
	skpz
	bsf 	LED

	clrf	f2				;we dont need f2
	movlw	d'100'
	movwf	y
	call	FXD248U			;compute delta complete, result in f2,f1,f0
	movf	f0,w
	addlw	0x01			;compensate for remainder (to be on the save side)
	movwf	caldelta
	movlw	high $
	movwf	PCLATH

    goto	hsm_humC
;------Submenu Humidity end
hsm_hum_end
	movlw	b'11000000'		;cursor at pos 1 at 2nd line
	call	Disp_send_conf
	strout	msg_mainmenu
	goto	hm_hum
;-----------Submenu Refill System----------------------------
hsm_ref
	movlw	b'11000000'		;cursor at pos 1 at 2nd line
	call	Disp_send_conf
	strout	msg_ref_menu
	movlw	b'10010000'		;cursor at pos 1 at 3rd line
	call	Disp_send_conf
	strout	msg_ref_usage
	movlw	b'11010000'		;cursor at pos 1 at 4th line
	call	Disp_send_conf
	strout	msg_empty
hsm_ref_1
	call	waitforrelease	;wait until no key pressed
	call	getkey			;get new status
	XORLW	0x04			;X? (exit)
	skpnz
	goto	hsm_ref_end
	;movf	keydata,w
	xorlw	0x03^0x04		;up?
	skpnz
	goto	hsm_refA		;yes, next menu entry
	;movf	keydata,w
	xorlw	0x05^0x03		;ok?
	skpz
	goto	hsm_ref_1
hsm_ref_11					;ok pressed
	bsf		confchange
	movlw	b'11010000'		;cursor at pos 1 at 4th line
	call	Disp_send_conf
	btfsc	refilluse
	goto	hsm_ref_12
	strout	msg_ref_off
hsm_ref_12
	btfss	refilluse
	goto	hsm_ref_13
	strout	msg_ref_on
hsm_ref_13
	call	waitforrelease
	call 	getkey
	xorlw	0x05			;ok?
	skpnz
	goto	hsm_ref			;yes, back to menu
	;movf	keydata,w
	xorlw	0x03^0x05		;up?
	skpnz	
	bsf		refilluse			;yes, set on
	;movf	keydata,w
	xorlw	0x06^0x03		;down?
	skpnz
	bcf		refilluse			;yes, then set off
	goto	hsm_ref_11
;--------Refill interval
hsm_refA
	movlw	b'00001100'		;set cursor blinking off
	call	Disp_send_conf
	movlw	b'10010000'		;cursor at pos 1 at 3rd line
	call	Disp_send_conf
	strout	msg_ref_int
	movlw	b'11010000'		;cursor at pos 1 at 4th line
	call	Disp_send_conf
	strout	msg_empty
hsm_refA_1
	call	waitforrelease	;wait until no key pressed
	call	getkey			;get new status
	XORLW	0x04			;X? (exit)
	skpnz
	goto	hsm_ref_end
	;movf	keydata,w
	xorlw	0x03^0x04		;up?
	skpnz
	goto	hsm_refB		;yes, next menu entry
	;movf	keydata,w
	xorlw	0x06^0x03		;down?
	skpnz
	goto	hsm_ref
	;movf	keydata,w
	xorlw	0x05^0x06		;ok?
	skpz
	goto	hsm_refA_1
	movlw	b'00001101'		;set cursor blinking
	call	Disp_send_conf
hsm_refA_11					;ok pressed
	bsf		confchange
	movlw	b'11010000'		;cursor at pos 0 at 4th line
	call	Disp_send_conf
	strout	msg_ref_intval
hsm_refA_12
	clrf	refilltimer		;we change interval, so start counting at 0
	movlw	b'11011001'		;cursor at pos 9 at 4th line
	call	Disp_send_conf
	movf	refint,w		;Interval to display
	call	bin2asc
	movf	asciiH,w
	call	Disp_send_data
	movf	asciiT,w
	call	Disp_send_data
	movf	asciiO,w
	call	Disp_send_data
	movlw	b'11011011'		;cursor at pos 11 at 4th line (for blink)
	call	Disp_send_conf
	call	waitforrelease
	call 	getkey
	xorlw	0x05			;ok?
	skpnz
	goto	hsm_refA		;yes, back to menu
	;movf	keydata,w
	xorlw	0x03^0x05		;up?
	skpz	
	goto	hsm_refA_13		;no
	movf	refint,w
	addlw	d'10'
	movwf	refint			;increment interval with 10
	xorlw	d'250'			;>240?
	skpz
	goto	hsm_refA_13
	movlw	d'240'			;yes, then set to 240
	movwf	refint
hsm_refA_13
	movf	keydata,w
	xorlw	0x06			;down?
	skpz	
	goto	hsm_refA_12		;no
	movlw	d'10'
	subwf	refint,f		;decrement interval with 10
	movf	refint,w
	xorlw	d'0'			;<10?
	skpz
	goto	hsm_refA_12
	movlw	d'10'			;yes, then set to 10
	movwf	refint
	goto	hsm_refA_12
;------------ Refill Duration
hsm_refB
	movlw	b'00001100'		;set cursor blinking off
	call	Disp_send_conf
	movlw	b'10010000'		;cursor at pos 1 at 3rd line
	call	Disp_send_conf
	strout	msg_ref_dur
	movlw	b'11010000'		;cursor at pos 1 at 4th line
	call	Disp_send_conf
	strout	msg_empty
hsm_refB_1
	call	waitforrelease	;wait until no key pressed
	call	getkey			;get new status
	XORLW	0x04			;X? (exit)
	skpnz
	goto	hsm_ref_end
	;movf	keydata,w
	xorlw	0x06^0x04		;down?
	skpnz
	goto	hsm_refA
	;movf	keydata,w
	xorlw	0x05^0x06		;ok?
	skpz
	goto	hsm_refB_1
	movlw	b'00001101'		;set cursor blinking
	call	Disp_send_conf
hsm_refB_11					;ok pressed
	bsf		confchange
	movlw	b'11010000'		;cursor at pos 0 at 4th line
	call	Disp_send_conf
	strout	msg_ref_durval
hsm_refB_12
	movlw	b'11011001'		;cursor at pos 9 at 4th line
	call	Disp_send_conf
	movf	refdur,w		;Duration to display
	call	bin2asc
	movf	asciiT,w
	call	Disp_send_data
	movf	asciiO,w
	call	Disp_send_data

	movlw	b'11011010'		;cursor at pos 10 at 4th line (for blink)
	call	Disp_send_conf
	call	waitforrelease
	call 	getkey
	xorlw	0x05			;ok?
	skpnz
	goto	hsm_refB		;yes, back to menu
	;movf	keydata,w
	xorlw	0x03^0x05		;up?
	skpz	
	goto	hsm_refB_13		;no
	incf	refdur,f		;increment Duration
	movf	refdur,w
	xorlw	d'11'			;>10?
	skpz
	goto	hsm_refB_13
	movlw	d'10'			;yes, then set to 10
	movwf	refdur
hsm_refB_13
	movf	keydata,w
	xorlw	0x06			;down?
	skpz	
	goto	hsm_refB_12		;no
	decf	refdur,f		;yes, decrement duration
	movf	refdur,w
	xorlw	d'0'			;<1?
	skpz
	goto	hsm_refB_12
	movlw	d'1'			;yes, then set to 1
	movwf	refdur
	goto	hsm_refB_12
;------Submenu Refill end
hsm_ref_end
	movlw	b'11000000'		;cursor at pos 1 at 2nd line
	call	Disp_send_conf
	strout	msg_mainmenu
	goto	hm_ref
;-----------Submenu Clockset----------------------------
hsm_clock
	bsf		confchange
	movf	hr,w
	movwf	f0
	movlw	high bcd2bin
	movwf	PCLATH
	call	bcd2bin
	movwf	tmphr
	movf	min,w
	movwf	f0
	call	bcd2bin
	movwf	tmpmin
	movlw	high hsm_clock
	movwf	PCLATH
	
	movlw	b'10010000'		;cursor at pos 0 at 3rd line
	call	Disp_send_conf
	strout	msg_clock_set
	movlw	b'10011010'		;cursor at pos 10 at 3rd line
	call	Disp_send_conf

	movlw	b'00001101'		;set cursor blinking
	call	Disp_send_conf
hsm_clock_11
	movlw	b'10011100'		;cursor at pos 12 at 3rd line
	call	Disp_send_conf
	movlw	':'			
	call	Disp_send_data
	movf	tmpmin,w		;Minutes to display
	call	bin2asc
	movf	asciiT,w
	call	Disp_send_data
	movf	asciiO,w
	call	Disp_send_data
hsm_clock_12
	movlw	b'10011010'		;cursor at pos 10 at 3rd line
	call	Disp_send_conf
	movf	tmphr,w			;Hours to display
	call	bin2asc
	movf	asciiT,w
	call	Disp_send_data
	movf	asciiO,w
	call	Disp_send_data
	movlw	b'10011011'		;cursor at pos 11 at 3rd line(for blink)
	call	Disp_send_conf
	call	waitforrelease
	call 	getkey
	xorlw	0x05			;ok?
	skpnz
	goto	hsm_clock_14	;yes, edit minutes
	;movf	keydata,w
	xorlw	0x03^0x05		;up?
	skpz	
	goto	hsm_clock_13	;no
	incf	tmphr,f			;increment hrs
	movf	tmphr,w
	xorlw	d'24'			;>23?
	skpz
	goto	hsm_clock_13
	movlw	d'23'			;yes, then set to 23
	movwf	tmphr
hsm_clock_13
	movf	keydata,w
	xorlw	0x06			;down?
	skpz	
	goto	hsm_clock_12	;no
	decf	tmphr,f			;yes, decrement hrs
	movf	tmphr,w
	xorlw	d'255'			;<0?
	skpz
	goto	hsm_clock_12
	movlw	d'0'			;yes, then set to 0
	movwf	tmphr
	goto	hsm_clock_12
hsm_clock_14					;****Edit Minutes
	movlw	b'10011101'		;cursor at pos 13 at 3rd line
	call	Disp_send_conf
	movf	tmpmin,w		;mins to display
	call	bin2asc
	movf	asciiT,w
	call	Disp_send_data
	movf	asciiO,w
	call	Disp_send_data
	movlw	b'10011110'		;cursor at pos 14 at 3rd line(for blink)
	call	Disp_send_conf
	call	waitforrelease
	call 	getkey
	xorlw	0x05			;ok?
	skpnz
	goto	hsm_clock_end	;yes, exit edit mode
	;movf	keydata,w
	xorlw	0x03^0x05		;up?
	skpz	
	goto	hsm_clock_15	;no
	incf	tmpmin,f		;increment mins
	movf	tmpmin,w
	xorlw	d'60'			;>59?
	skpz
	goto	hsm_clock_15
	movlw	d'59'			;yes, then set to 59
	movwf	tmpmin
hsm_clock_15
	movf	keydata,w
	xorlw	0x06			;down?
	skpz	
	goto	hsm_clock_14	;no
	decf	tmpmin,f		;yes, decrement mins
	movf	tmpmin,w
	xorlw	d'255'			;<0?
	skpz
	goto	hsm_clock_14
	movlw	d'0'			;yes, then set to 0
	movwf	tmpmin
	goto	hsm_clock_14

hsm_clock_end
	movlw	high bin2bcd
	movwf	PCLATH
	movf	tmphr,w			;copy new time to live registers
	call	bin2bcd
	movwf	hr				;...hours
	movf	tmpmin,w
	call	bin2bcd
	movwf	min				;...minutes

	clrf	sec				;and we want to start a new minute when set
	call	rtc_brst_wr		;write to rtc
	call	disp_time		;and update display of time

	movlw	high hsm_clock_end
	movwf	PCLATH
	
	movlw	b'00001100'		;set cursor blinking off
	call	Disp_send_conf
	goto	hm_clock		;ready, go back



;---------------------------------------------------------------	
;key handling
;returns status of keys in w
;0= nothing pressed
;6= down /1
;5= ok /2
;4= X /3
;3= up /4
;2= P
;1= Menu
;---------------------------------------------------------------
;doesn't wait for anything, just checks current status
keycheck
	movf	PORTB,w
	movwf	keydata
	swapf	keydata,f			;get status of port B und 
	;rrf		keydata,f			;shift bits 4-7 to bit  0-3
	movlw	b'00000111'			;finally mask Bit 0 and 1
	andwf	keydata,f
	movlw	b'00000111'
	xorwf	keydata,f			;and set them to positive logic
								;because keys are active=low
	movf	keydata,w			;save to w for calling routine
	return
;waits for key to be pressed
getkey
	bsf		Backlight
	clrf	bltimer
	call 	keycheck
	skpnz						;if no key pressed
	goto 	getkey				;check again
	return						;otherwise return to caller
;waits for release of any key
waitforrelease
	call 	Delay10ms			;debounce
	call	keycheck
	movf	keydata,w
	skpz						;if zero then enough
	goto	waitforrelease
	call 	Delay10ms			;debounce
	return
;---------------------------------------------------------------	
;String Routines for using the LCD Display
;---------------------------------------------------------------
;disp_string
;sends string at look_hi/look_lo to display at current cursor position
;ends at 0x00 (end criteria)
Disp_string
	call 	lookup			;char of string to w
	;clrf	PCLATH			;we were in pgm-mem Page1, so reset to Page 0
    movwf   tmp             ;save returned char
    movlw   high $          ;make sure we're in the right pgm page (cause strings could be in pgm page 0 now)
    movwf   PCLATH
    movf    tmp,w
	andlw	0xff			;set Z-Flag if zero
	skpnz					;if zero (last char)
	return					;end routine
	call Disp_send_data
	goto Disp_string


;lookup
;jump to address in look_hi/look_lo which is a RETLW
;increment pointer afterwards
lookup
	movf	look_hi,w		;set PCLATH
	movwf	PCLATH
	movf	look_lo,w		;and get PCL
	incf	look_lo,f		;next position
	skpnz
	incf	look_hi,f		;also if overflow
	movwf	PCL				;now jump
;---------------------------------------------------------------	
;Display Routines for using the LCD Display
;---------------------------------------------------------------

;clear display and return cursor home
Disp_clear
	movlw	b'00000001'		;Clear Display
	call 	Disp_send_conf
	return
;Initialize Display	
Disp_Init
	call 	Delay100ms		;Wait for Display to be ready (first Start)
	
	movlw	b'00000010'		;Init to 4-Bit Mode (this time in 8-Bit Mode)
	movwf	DISPPort
	bsf		DISPE	;pulse Enable
	nop
	bcf		DISPE	
	call	Delay10ms
	;once more...
	movlw	b'00000010'		;Init to 4-Bit Mode (this time in 8-Bit Mode)
	movwf	DISPPort
	bsf		DISPE	;pulse Enable
	nop
	bcf		DISPE	
	call 	Delay10ms
	;once more...
	movlw	b'00000010'		;Init to 4-Bit Mode (this time in 8-Bit Mode)
	movwf	DISPPort
	bsf		DISPE	;pulse Enable
	nop
	bcf		DISPE	
	call 	Delay10ms
	
	movlw	b'00101000'		;Display Init two rows (now in 4-Bit Mode)
	call 	Disp_send_conf
	call	Delay10ms
	movlw	b'00001100'		;Display on, Cursor off, No Blinking
	call 	Disp_send_conf
	call 	Delay10ms
	movlw	b'00000001'		;Clear Display
	call 	Disp_send_conf
	call	Delay10ms
	movlw	b'00000110'		;cursor auto-increment
	call	Disp_send_conf
	call	Delay10ms
	;movlw	high Disp_init_chars	;cause this routine is in Page 1
	;movwf	PCLATH
	;call	Disp_init_chars
	;clrf	PCLATH			;we were in pgm-mem Page1, so reset to Page 0
	return

;----------------------------------	
;Send Config-Data to Display (RS=0)
Disp_send_conf
	;bcf		tmpGIE,0
	;btfss	INTCON,GIE
	;goto	disp_send_conf_1
	;bsf		tmpGIE,0		;remember State of GIE
	bcf		INTCON,GIE		;we don't want interrupts now
	btfsc	INTCON,GIE		;make sure it is really cleared
	goto	$-2
;disp_send_conf_1

	movwf	Dispdata		;save w
	movlw	0xC0			;this is because we need to retain status
							;of upper nibble (SDA and SCL at Bit 6&7)
	andwf	DISPPort,f		;mask upper nibble, set lower nibble to 0
	swapf	Dispdata,w		;first we send upper nibble
	andlw	0x0f			;mask it
	iorwf	DISPPort,f		;output it
	nop
	bsf		DISPE	;pulse Enable
	nop
	bcf		DISPE
	movlw	0xC0
	andwf	DISPPort,f		;mask upper nibble, set lower nibble to 0
	movf	Dispdata,w		;and once again with lower nibble
	andlw	0x0f
	iorwf	DISPPort,f
	nop
	bsf		DISPE
	nop
	bcf		DISPE
	call 	Delay100us
	;btfsc	tmpGIE,0			;restore GIE
	bsf		INTCON,GIE
	return

;--------------------------
;Send Data to Display (RS=1)
Disp_send_data
	;bcf		tmpGIE,0
	;btfss	INTCON,GIE
	;goto	disp_send_data_1
	;bsf		tmpGIE,0		;remember State of GIE
	bcf		INTCON,GIE		;we don't want interrupts now
	btfsc	INTCON,GIE		;make sure it is really cleared
	goto	$-2
;disp_send_data_1

	movwf	Dispdata		;save w
	movlw	0xC0			;this is because we need to retain status
							;of upper nibble (SDA and SCL at Bit 6&7)
	andwf	DISPPort,f		;mask upper nibble, set lower nibble to 0
	swapf	Dispdata,w		;first we send upper nibble
	andlw	0x0f			;mask it
	iorwf	DISPPort,f		;output it
	bsf		DISPRS		;RS=high for Data write
	nop
	nop
	nop
	bsf		DISPE	;pulse Enable
	nop
	bcf		DISPE
	movlw	0xC0			;this is because we need to retain status
							;of upper nibble (SDA and SCL at Bit 6&7)
	andwf	DISPPort,f		;mask upper nibble, set lower nibble to 0
	movf	Dispdata,w		;and once again with lower nibble
	andlw	0x0f
	iorwf	DISPPort,f
	bsf		DISPRS		;RS=high for Data write
	nop
	nop
	nop
	bsf		DISPE
	nop
	bcf		DISPE
	bcf		DISPRS			;just to be sure
	call 	Delay100us
	;btfsc	tmpGIE,0			;restore GIE
	bsf		INTCON,GIE
	return	
;---------------------------------------------------------------
;bin2asc
;converts 8-bit value in w to 3-digit ascii value
;value is in global variables asciiO to asciiH
;---------------------------------------------------------------
bin2asc
	movwf	asciiO		;start with value in asciiO
	movlw	a'0'
	movwf	asciiT
	movwf	asciiH
do100s
	movlw	d'100'
	subwf	asciiO,w
	bnc		do10s
	movwf	asciiO
	incf	asciiH,f
	goto	do100s
do10s
	movlw	d'10'
	subwf	asciiO,w
	bnc		adjust
	movwf	asciiO
	incf	asciiT,f
	goto	do10s
adjust
	movlw	a'0'
	addwf	asciiO,f
	return
;--------------------------------------------------------------------
;Delay-Subroutines (for fosc=4 MHz!!!)
;--------------------------------------------------------------------
Delay100us					;Delays ca. 100us
	movlw	0x1F
	movwf	d1
Delay100us_1
	decfsz	d1, f
	goto	Delay100us_1
	goto	$+1
	return

;-----
Delay10ms					;Delays ca.10ms
	movlw	0xCE
	movwf	d1
	movlw	0x08
	movwf	d2
Delay10ms_1
	decfsz	d1, f
	goto	$+2
	decfsz	d2, f
	goto	Delay10ms_1
	goto	$+1
	nop
	return

;-----
Delay100ms					;Delays ca.100ms
	movlw	0x1E
	movwf	d1
	movlw	0x4F
	movwf	d2
Delay100ms_1
	decfsz	d1, f
	goto	$+2
	decfsz	d2, f
	goto	Delay100ms_1
	goto	$+1
	nop
	return

;---------------------------------------------------------------------------
; Message definitions for display (2)
;---------------------------------------------------------------------------
msg_hum_menu	dt	"-Humidity Sensor-",0x00
msg_hum_usage	dt	"|Sensor on/off >",0x00
msg_hum_on		dt	"Sensor On       ",0x00
msg_hum_off		dt	"Sensor Off      ",0x00
msg_hum_tres	dt	"<Sens. treshold>",0x00
msg_hum_tresval	dt	"Treshold XX%    ",0x00
msg_hum_int		dt	"<Sens. interval>",0x00
msg_hum_intval	dt	"Interval XXmin  ",0x00
msg_hum_calib	dt	"<Calibrate     |",0x00
msg_hum_caldry	dt	"Sensor dry(ok,X)",0x00
msg_hum_calwet	dt	"Sensor wet(ok,X)",0x00
msg_hum_hz		dt  " Hz  ",0x00

msg_ref_menu	dt	"-Refill System--",0x00
msg_ref_usage	dt	"|Refill on/off >",0x00
msg_ref_on		dt	"Refill Sys on   ",0x00
msg_ref_off		dt	"Refill Sys off  ",0x00
msg_ref_int		dt	"<Ref. interval >",0x00
msg_ref_intval	dt	"Interval XXXmin ",0x00
msg_ref_dur		dt	"<Max. duration |",0x00
msg_ref_durval	dt	"Duration XXmin  ",0x00

msg_clock_set	dt	"New Time: XX:XX ",0x00

msg_empty		dt	"                ",0x00

msg_run			dt	"RUN",0x00
msg_norun		dt	"   ",0x00

	
	end